# -*- coding: utf-8 -*-
###########################################################################
# Eole NG - 2007  
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr 
#  
#  Outil de gestion de set de machine
#       
###########################################################################
""" Action d'evaluation d'url """
from twisted.python import log
from ead2.backend.lib.action import * # import des actions et des types
from ead2.backend.actions import tools
from ead2.backend.actions.amon import amon_tools as t
from ead2.backend.actions.lib.ipset import group_manager, time_manager, rule_generator, tools as ipset_tools
from ead2.lib.error import MissingKey, MissingValue
from os.path import isfile, join
from sets import Set
used_templates = ['main', 'form', 'accordion', 'formselect']
from creole.parsedico import parse_dico

def test_ip_pedago(ip):
    """ teste si une ip appartient bien au sous-réseau pedago """
    pedago = ipset_tools.get_pedago()[0]
    print "Adresse pedago : %s"%pedago
    if pedago != None:
        subnet = '.'.join(pedago.split('.')[:3])
        print "ip %s, subnet %s"%(ip, subnet)
        if ip.startswith(subnet):
            return True
    return False

def compare_time_range(tstart, tstop):
    """ compare une heure de début et de fin de plage et renvoie True si dans l'ordre """
    tstart = tstart.split(':')
    tstop = tstop.split(':')
    if int(tstart[0]) > int(tstop[0]):
        return False
    elif int(tstart[0]) == int(tstop[0]):
        if int(tstart[1]) >= int(tstop[1]):
            return False
    return True

def get_time_range_table():
    """ renvoie une table [(0,0), (1,0), (2,0)...(3,0)]"""
    table = [('0', '0')]
    for i in range(1, 9):
        table.append((str(i)+'h',str(0)))
    table.append(('9h', '1'))

    for i in range(0,9):
        table.append((str(i)+'h',str(1)))
    table.append(('9h', '2'))

    for i in range(3):
        table.append((str(i)+'h',str(2)))
    table.append(('3h', '0'))
    return table

def set_ipset_rules():
    """ éxécute la génération de règle ipset """
    generator = rule_generator.RuleGenerator()
    generator.create_ip_sets()

def set_iptables_rules():
    """ relance bastion (remise à zéro des règles iptables)
    """
    #FIXME: on peut faire évoluer les performances de l'outil en testant l'existence des règles pour éviter les bastion restart inutil...
    t.bastion_restart()
    
def get_close_btn():
    """ renvoie la description du bouton de vidage de la balise ip_set_workspace(zone de travail) """
    href = "javascript:setVoid('ip_set_workspace');"
    title = "Fermer la section"
    libelle = "Fermer"
    icone = "/image/supprimer.gif"
    _class = "ip_set_close_btn"
    return dict(href=href,
                title=title,
                libelle=libelle,
                icone=icone,
                _class=_class)


class GroupeMachine(Action):
    """ gestion des sets ipset de machine """
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name', 'role'])
    name = 'groupe_machine'
    libelle = "Groupe de machine"
    description = "Gère des groupes de machine"
    category = "Configuration"
    request = Dict(default={}, doc="arguments de la requete en cour cote frontend",
                                keys=['server', 'action', 
                                        'delete', 'menu', 
                                        'active', 'unactive', 
                                        'active_time', 'unactive_time',
#                                        'ip_set', 'policy'
                                        ])
    
    def execute(self):
        """ renvoie les données pour la mise en forme des actions 
        """
        self.params, self.server_nb = tools.get_request(self.request)
        if self.params.has_key('delete'):
            try:
                datas = self.delete_set()
                datas['toexec'] = tools.make_js_link(self.server_nb, self.name, 
                                    confirm=True, balise='ip_set_menu', menu='ok')
            except Exception, mess:
                import traceback
                traceback.print_exc()
                datas = {'message':mess.message}
            return self.send_frag(datas, template="amon_groupe_machine", templates=used_templates)

        elif self.params.has_key('active') or self.params.has_key('unactive'):
            try:
                if self.params.has_key('active'):
                    datas = self.active_set()
                else:
                    datas = self.active_set(False)
                datas['toexec'] = tools.make_js_link(self.server_nb, self.name,
                                    confirm=True, balise='ip_set_menu', menu='ok')
            except Exception, mess:
                import traceback
                traceback.print_exc()
                datas = {'message':mess.message}
            return self.send_frag(datas, template="amon_groupe_machine", templates=used_templates)

        elif self.params.has_key('active_time') or self.params.has_key('unactive_time'):
            try:
                if self.params.has_key('active_time'):
                    datas = self.active_time()
                else:
                    datas = self.active_time(False)
                datas['toexec'] = tools.make_js_link(self.server_nb, self.name,
                                    confirm=True, balise='ip_set_menu', menu='ok')
            except Exception, mess:
                import traceback
                traceback.print_exc()
                datas = {'message':mess.message}
            return self.send_frag(datas, template="amon_groupe_machine", templates=used_templates)

#        elif self.params.has_key('policy'):
#            try:
#                datas = self.set_policy()
#                datas['toexec'] = tools.make_js_link(self.server_nb, self.name, confirm=True, balise='ip_set_menu', menu='ok')
#            except Exception, msg:
#                datas = dict(message=msg.message)
#            return self.send_frag(datas, template="amon_groupe_machine", templates=used_templates)
        elif self.params.has_key('menu'):
            datas = self._get_ip_sets()
            return self.send_frag(datas, template="amon_groupe_machine", templates=used_templates)
        result = {'titre':self.libelle}
        result.update(self._get_ip_set_create_btn())
        if self.user_description['role'] == 'admin':
            result.update(self._get_config_pedago_btn())
        result['toexec'] = tools.make_js_link(self.server_nb, self.name, confirm=True, balise='ip_set_menu', menu='ok')
        return self.send_all(result, template="amon_groupe_machine", templates=used_templates)

    def _get_ip_set_create_btn(self):
        href = tools.make_js_link(self.server_nb, 'groupe_machine_create', balise='ip_set_workspace')
        title = "créer un groupe de machine"
        libelle = "Nouveau groupe de machine"
        icone = ""
        return dict(new_ip_set={'href':href, 'title':title, 'libelle':libelle, 'icone':icone})

    def _get_config_pedago_btn(self):
        """ renvoie la description du bouton d'appel du formulaire de configuration du sous-réseau pédago 
        """
        href = tools.make_js_link(self.server_nb, 'groupe_machine_config_pedago', balise='ip_set_workspace')
        title = "Configurer le sous-réseau pédago"
        libelle = "Configurer le sous-réseau pédagogique"
        icone = ""
        return dict(config_pedago={'href':href, 'title':title, 'libelle':libelle, 'icone':icone})

    def _get_policy_form(self, ip_set):
        """ renvoie la description du formulaire de choix de la politique de filtrage """
        options = [{'name':column, 'libelle':column}for column in t.columns_list]
        select_policy = {'onchange':tools.make_js_link(self.server_nb, self.name, code=True, balise="ip_set_menu", 
                                                        policy='this.options[this.selectedIndex].value', ip_set="'%s'"%ip_set['name']),
                         'name':'select_policy',
                         'options':[{'name':'', 'options':options}]}
        return select_policy


    def _get_ip_sets(self):
        """ renvoie la description pour la mise en forme de set d'ip """
        def sort_func(a,b):
            """ fonction de classement selon le nom des groupes """
            if a['name'] > b['name']: return 1
            else: return -1

        ip_sets = group_manager.get_all_groups()
        ip_sets.sort(sort_func)
        
        # restrictions professeur
        if self.user_description['role'] != 'admin':
            ip_sets = [ip_set for ip_set in ip_sets if test_ip_pedago(ip_set['ip_from'])]
        
        ip_set_list_libelles = ("Groupes de machine", "horaires", "Interdiction de web", "Interdiction de web selon horaire", 
                                "suppression",)# 'politique <br />optionnelle')
        sets = []
        for ip_set in ip_sets:
            libelle = "<b>%s</b> Ips: %s à %s"%(ip_set['name'], ip_set['ip_from'], ip_set['ip_to'])
            horaire_btn = self.make_ip_set_time_btn(ip_set)
            suppr_btn = self.make_ip_set_del_btn(ip_set)
            active_btn = self.make_active_btn(ip_set)
            active_time = self.make_active_time_btn(ip_set)
#            select_policy = self._get_policy_form(ip_set)
            sets.append((libelle, horaire_btn, active_btn, active_time, suppr_btn))#, self._get_policy_form(ip_set)))
        return dict(ip_sets=sets, ip_set_list_libelles=ip_set_list_libelles)

    def make_ip_set_del_btn(self, ip_set):
        """ bouton supprimer """
        libelle = ""
        _href = tools.make_js_link(self.server_nb, self.name, confirm=True, balise='ip_set_del_msg',
                                        delete=ip_set['name'])
        href = tools.make_confirm_link("Êtes-vous sûr de vouloir supprimer le groupe de machine %s?"%ip_set['name'], _href)
        title = "Supprimer le groupe de machine %s"%ip_set['name']
        icone = "/image/supprimer.gif"
        _class = "amon_ip_set_del_btn"
        if self.user_description['role'] != 'admin':
            href = ''
            title = ''
            icone = ''
        return dict(libelle=libelle, href=href, title=title, icone=icone, _class=_class)

    def make_ip_set_active_btn(self, ip_set):
        """ bouton activer """
        libelle = "Mod"
        href = tools.make_js_link(self.server_nb, 'groupe_machine_service', confirm=False, 
                                    balise='ip_set_workspace', ip_set=ip_set['name'])
        title = "Associer des services au groupe %s"%ip_set['name']
        icone = ""
        _class = "amon_ip_set_btn"
        return dict(libelle=libelle, href=href, title=title, icone=icone, _class=_class)
    
    def make_ip_set_time_btn(self, ip_set):
        """ bouton configuration horaire """
        libelle = ""
        href = tools.make_js_link(self.server_nb, 'groupe_machine_horaire', confirm=False, 
                                    balise='ip_set_workspace', ip_set=ip_set['name'])
        title = "Définir des plage horaire pour %s"%ip_set['name']
        icone = "/image/amon/time.png"
        _class = "amon_ip_set_btn"
        return dict(libelle=libelle, href=href, title=title, icone=icone, _class=_class)

    def make_active_btn(self, ip_set):
        """ bouton d'activation l'interdiction web """
        libelle = ""

        if (ip_set['active'] == '0') or (ip_set['active'] == '1' and ip_set['time'] == '1'):
            href = tools.make_js_link(self.server_nb, self.name, confirm=False, balise='ip_set_del_msg',
                                    active=ip_set['name'])
            title = "Activer l'interdiction web sur ce groupe de machine"
            icone = "/image/amon/enable.png"
        else:
            href = tools.make_js_link(self.server_nb, self.name, confirm=False, balise='ip_set_del_msg',
                                                        unactive=ip_set['name'])
            title = "Désactiver l'interdiction web sur ce groupe de machine"
            icone = "/image/amon/disable.png"

        _class = "amon_ip_set_btn"
        return dict(libelle=libelle, href=href, title=title, icone=icone, _class=_class)

    def make_active_time_btn(self, ip_set):
        """ bouton d'activation de la gestion du temp """
        libelle = ""

        if (ip_set['time'] == '0') or (ip_set['active'] == '0' and ip_set['time'] == '1'):
            href = tools.make_js_link(self.server_nb, self.name, confirm=False, balise='ip_set_del_msg',
                                         active_time=ip_set['name'])
            title = "Activer l'interdiction horaire sur ce groupe de machine"
            icone = "/image/amon/enable.png"
        else:
            href = tools.make_js_link(self.server_nb, self.name, confirm=False, balise='ip_set_del_msg',
                                                             unactive_time=ip_set['name'])
            title = "Désactiver l'interdiction horaire sur ce groupe de machine"
            icone = "/image/amon/disable.png"

        _class = "amon_ip_set_btn"
        return dict(libelle=libelle, href=href, title=title, icone=icone, _class=_class)

    def delete_set(self):
        """ supprime un set d'ip ipset """
        todel = self.params['delete'][0]
        group = group_manager.get_group(todel)
        if group_manager.del_group(todel):
            if group['active'] == '1': ## des règles iptables s'applique au groupe de machine
                set_iptables_rules()
            set_ipset_rules()
            return dict(message="Le groupe de machine %s a bien été supprimé."%todel)
        else:
            raise Exception, "Erreur: une erreur est survenue à la suppression du groupe de machine %s."%todel

    def active_set(self, active=True):
        """ active les restrictions sur un groupe de machine """
        if active:
            toactive = self.params['active'][0]
            if group_manager.active(toactive) and group_manager.unactive_time(toactive):
                set_iptables_rules()
                return dict(message="Le groupe de machine %s est désormais interdit de navigation web."%toactive)
            else:
                raise Exception, "Erreur: une erreur est survenue à l'activation du groupe de machine %s."%toactive
        else:
            tounactive = self.params['unactive'][0]
            if group_manager.unactive(tounactive):
                set_iptables_rules()
                return dict(message="Le groupe de machine %s peut désormais naviguer librement."%tounactive)
            else:
                raise Exception, "Erreur: une erreur est survenue à la désactivation du groupe de machine %s."%tounactive

    def active_time(self, active=True):
        """ active/désactive les interdictions horaires """
        if active:
            toactive = self.params['active_time'][0]
            if group_manager.active_time(toactive) and group_manager.active(toactive):
                set_iptables_rules()
                return dict(message="Le groupe de machine %s est désormais interdit de navigation web selon les horaires spécifiés."%toactive)
            else:
                raise Exception, "Erreur: une erreur est survenue à l'activation des horaires du groupe de machine %s."%toactive
        else:
            tounactive = self.params['unactive_time'][0]
            if group_manager.unactive_time(tounactive) and group_manager.unactive(tounactive):
                set_iptables_rules()
                return dict(message="Le groupe de machine %s n'est plus soumis à des interdictions horaires."%tounactive)
            else:
                raise Exception, "Erreur: une erreur est survenue à la désactivation des horaires du groupe de machine %s."%tounactive

#    def set_policy(self):
#        """ définit la politique optionnelle à appliquer à un groupe de machine """
#        set = self.params['ip_set'][0]
#        policy = self.params['policy'][0]
#        # FIXME: définit la plage d'ip du groupe dans le fichier ip_groups de dansguardian
#        return dict(message = "La politique %s a été associé au groupe %s."%(policy, set))



class GroupeMachineCreate(Action):
    """ gestion des sets ipset de machine """
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name', 'role'])
    name = 'groupe_machine_create'
    libelle = "Création de groupe de machine"
    description = "Crée un groupes de machine"
    category = None
    request = Dict(default={}, doc="arguments de la requete en cour cote frontend",
                                keys=['server', 'action'])
    form_result = Dict(default={}, doc="retour formulaire de la requete en cour coté frontend",
                                    keys = ['set_create'])
    def execute(self):
        """ renvoie les données pour la mise en forme des actions 
        """
        self.params, self.server_nb = tools.get_request(self.request)
        if self.form_result != {}:
            if self.form_result.has_key('set_create'):
                try:
                    datas = self.create_set()
                    datas['toexec'] = tools.make_js_link(self.server_nb, 'groupe_machine', 
                                                    confirm=True, balise='ip_set_menu', menu='ok') 
                except Exception, mess:
                    datas = {'message':mess.message}
                self.form_result = {}
                return self.send_frag(datas, template="amon_groupe_machine", templates=used_templates)
        result = self._get_form()
        result['libelle'] = self.libelle
        result['close'] = get_close_btn()
        return self.send_all(result, template="amon_groupe_machine", templates=used_templates)

    def _get_form(self):
        """ renvoie les données pour la mise en forme du formulaire de création de groupe de machine"""
        new_set_name = {'name':'set_name', 'libelle':'nom du groupe'}
        ip_from = {'name':'ip_from', 'libelle':"début de la plage d'ip"}
        ip_to = {'name':'ip_to', 'libelle':"fin de la plage d'ip"}
        return dict(new_set_name=new_set_name,
                    ip_from=ip_from,
                    ip_to=ip_to,
                    validate=self._get_validate_btn())

    def _get_validate_btn(self):
        """ renvoie les données pour la mise en forme du bouton valider """
        href = tools.make_form_link(self.server_nb, self.name, True, ['set_create'], balise='set_create_msg')
        title = "Valider la création de ce groupe"
        libelle = "Valider"
        icone = "/image/ok.gif"
        return dict(href=href,
                    title=title,
                    libelle=libelle,
                    icone=icone)

    def create_set(self):
        """ crée un set ipset """
        resultat = tools.format_form_result(self.form_result['set_create'])
        if Set(['set_name', 'ip_from', 'ip_to']) != Set(resultat.keys()):
            log.err("groupe_machine_create : erreur, il manque des données à la création de groupe.")
            raise MissingKey, "Erreur: il manque des données à la création de groupe de machine."
        name = resultat['set_name']
        ip_from = resultat['ip_from']
        ip_to = resultat['ip_to']
        
        # restriction professeur
        if self.user_description['role'] != 'admin':
            if not test_ip_pedago(ip_from) or not test_ip_pedago(ip_to):
                raise Exception, "Erreur: Vous n'êtes pas autorisé à gérer ces ips" 

        if group_manager.create_group(name, ip_from, ip_to):
            set_ipset_rules()
            return dict(message="Le groupe de machine %s a été créé.\\nIl recouvre la plage d'ip %s - %s."%(name, ip_from, ip_to))
        else:
            log.err("groupe_machine_create : erreur au lancement de manager.create_ip_set.")
            raise Exception, "Erreur: une erreur s'est produite à la création du set %s"%name


class GroupeMachineConfigPedago(Action):
    """ configuration de l'interface pedago """
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name', 'role'])
    name = 'groupe_machine_config_pedago'
    libelle = "Configuration du sous-réseau pédagogique"
    description = "Configurer le sous-réseau pédagogique"
    category = None
    request = Dict(default={}, doc="arguments de la requete en cour cote frontend",
                                keys=['server', 'action'])
    form_result = Dict(default={}, doc="retour formulaire de la requete en cour coté frontend",
                                    keys = ['pedago'])
    def execute(self):
        """ renvoie les données pour la mise en forme des actions 
        """
        self.params, self.server_nb = tools.get_request(self.request)
        
        if self.form_result != {}:
            try:
                datas = self.configure_pedago()
            except Exception, msg:
                datas = {'message':msg.message}
            self.form_result = {}
            return self.send_frag(datas, template="amon_groupe_machine", templates=used_templates)

        result = {'libelle':"Configurer l'interface du sous-réseau pédago"}
        result.update(self._get_form())
        result['close'] = get_close_btn()
        return self.send_all(result, template="amon_groupe_machine", templates=used_templates)

    def _get_form(self):
        """ renvoie la description du formulaire de configuration de l'interface du réseau pédagogique 
        """
        d = parse_dico()

        num_interface = int(d['nombre_interfaces'])
        interfaces = ['eth%s'%i for i in range(1, num_interface)]
        interfaces = [(interface, d['nom_machine_%s'%interface]) for interface in interfaces]

        options = [{'libelle': "%s (%s)"%(interface, libelle), 'name':interface}for interface, libelle in interfaces]
        test = True
        for opt in options:
            if opt['name'] == ipset_tools.get_pedago()[1]:
                opt['default'] = 'ok'
                test = False
        if test:
            options.append({'libelle':'Aucune', 'name':'', 'default':'ok'})

        interface = {'libelle': "Choisir une interface comme réseau pédagogique (dans le cas d'administrateur pédagogique)",
                     'name':'interface',
                     'options':[{'name':"interfaces disponibles sur l'amon",
                                 'options':options,
                                 'size':'5'}]}
        return dict(interface=interface,
                    validate = self._get_validate_btn())
    
    def _get_validate_btn(self):
        """ renvoie les données pour la mise en forme du bouton valider """
        href = tools.make_form_link(self.server_nb, self.name, True, ['pedago'], balise='config_pedago_msg')
        title = "Valider la configuration l'interface pédagogique"
        libelle = "Valider"
        icone = "/image/ok.gif"
        return dict(href=href,
                    title=title,
                    libelle=libelle,
                    icone=icone)
    
    def configure_pedago(self):
        """ configure le réseau pédagogique niveau ead2 pour la gestion des groupes de machine """
        resultat = tools.format_form_result(self.form_result['pedago'])
        interface = resultat['interface']
        
        d = parse_dico()
        ip = d['adresse_ip_%s'%interface]
        ipset_tools.config_pedago(ip, interface)
        return dict(message = "Le réseau pédagogique a été configuré comme fonctionnant sur l'interface %s (ip %s)"%(interface, ip))

class GroupeMachineHoraire(Action):
    """ gestion des liens des horaires des groupes de machine """
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name', 'role'])
    name = 'groupe_machine_horaire'
    libelle = "Application de plage horaire"
    description = "Définir des horaires d'application pour les restrictions"
    category = None
    request = Dict(default={}, doc="arguments de la requete en cour cote frontend",
                        keys=['server', 'action', 'ip_set', 'todel'])
    form_result = Dict(default={}, doc="retour formulaire de la requete en cour coté frontend",
                        keys = ['ip_set_time_range', 'ip_set_current_set', 'ip_set_time_range_copy'])
    def execute(self):
        """ renvoie les données pour la mise en forme des actions 
        """
        self.params, self.server_nb = tools.get_request(self.request)
        self.group = None
        if self.form_result != {}:
            if self.form_result.has_key('ip_set_time_range'):
                try:
                    datas = self.set_time_range()
                    datas['toexec'] = tools.make_js_link(self.server_nb, self.name, balise='ip_set_workspace', confirm=False, ip_set=self.group)
                except Exception, msg:
                    import traceback
                    traceback.print_exc()
                    datas = {'message':msg.message}
                self.form_result = {}
                return self.send_frag(datas, template="amon_groupe_machine", templates=used_templates)

            if self.form_result.has_key('ip_set_time_range_copy'):
                try:
                    datas = self.copy_time_range()
                    datas['toexec'] = tools.make_js_link(self.server_nb, self.name, balise='ip_set_workspace', confirm=False, ip_set=self.group)
                except Exception, msg:
                    import traceback
                    traceback.print_exc()
                    datas = {'message':msg.message}
                self.form_result = {}
                return self.send_frag(datas, template="amon_groupe_machine", templates=used_templates)

        if self.params.has_key('todel'):
            try:
                datas = self.del_time_range()
                datas['toexec'] = tools.make_js_link(self.server_nb, self.name, balise='ip_set_workspace', confirm=False, ip_set=self.group)
            except Exception, msg:
                import traceback
                traceback.print_exc()
                datas = {'message':msg.message}
            self.params.pop('todel')
            return self.send_frag(datas, template="amon_groupe_machine", templates=used_templates)

        if self.params.has_key('ip_set'):
            self.group = self.params['ip_set'][0]
            self.group_datas = self._get_group_datas()
        elif self.group == None:
            datas = {'toexec': tools.make_js_link(self.server_nb, 'groupe_machine')}
            return self.send_frag(datas, template="amon_groupe_machine", templates=used_templates)
        result = {'ip_set_name': self.group}
        result.update(self._get_time_range())
        result.update(self._get_copy_time_range_form())
        result['close'] = get_close_btn()
        return self.send_all(result, template="amon_groupe_machine", templates=used_templates)
    
    def _get_group_datas(self):
        """ renvoie les données pour la mise en forme d'un groupe """
        datas = group_manager.get_group(self.group)
        return datas

    def _get_time_range(self):
        """ renvoie les données pour la configuration de plage horaire """
        hours = time_manager.get_time_list()
        jours_datas = {}
        for jour in time_manager.get_days_list():
            horaires = time_manager.get_schedules(self.group, jour)
            closed_hours = []
            btns = []

            # fermé par defaut
            for i in range(48):
                closed_hours.append('0')
                btns.append('')
        
            libelle = "<b>Interdiction de navigation web:</b>"
            for horaire in horaires:
                libelle += "<br />de %s à %s"%(str(horaire['hdeb']), str(horaire['hfin']))
                deb = hours.index(horaire['hdeb'])
                end = hours.index(horaire['hfin'])
                middle = (deb+end)/2
                for i in range(deb, end):
                    closed_hours[i] = '1'

                btns[middle] = self._get_del_time_range_btn(jour, horaire)
                if middle/2 == (middle-1)/2:
                    btns[middle-1] = 'void'
                else:
                    btns[middle+1] = 'void'
            if horaires == []:
                libelle += "<br />Aucune restriction n'est appliquée le %s"%jour
            jours_datas[jour] = (closed_hours, btns, libelle)


        ip_set_name = {'name':'ip_set_name', 'default_value':self.group}
        start_hours = t.get_hours()
        end_hours = t.get_hours()
        days = t.get_days()

        tstart_select = {'name':'tstart', 
                         'libelle':'Début de plage',
                         'options':[{'name':'debut', 'options':start_hours}]}

        tstop_select = {'name':'tstop', 
                        'libelle':'Fin de plage',
                        'options':[{'name':'fin', 'options':end_hours}]}

        days_select = {'name':'day',
                       'libelle':"Choix du (des jours)",
                       'options':[{'name':'', 'options':days}],
                        'multi':'ok',
                        'size':'7'}

        return dict(ip_set_name_input=ip_set_name,
                    tstart_select=tstart_select,
                    tstop_select=tstop_select,
                    days_select=days_select,
                    time_range_table=get_time_range_table(),
                    jours_datas=jours_datas,
                    jours=time_manager.get_days_list(),
                    validate_time_range=self._get_validate_time_range_btn(),
                    )

    def _get_validate_time_range_btn(self):
        """ renvoie la decsription du bouton de validation pour la définition de plage de temp """
        href = tools.make_form_link(self.server_nb, self.name, True, ['ip_set_time_range', 'ip_set_current_set'], balise='ip_set_time_range_msg')
        title = "Définir ces horaires pour ce groupe de machine."
        libelle = "Valider"
        icone = '/image/ok.gif'
        return dict(href=href, title=title, libelle=libelle, icone=icone)

    def _get_del_time_range_btn(self, jour, plage):
        """ renvoie la description du bouton de suppression de plage horaire """
        icone = "/image/supprimer.gif"
        href = tools.make_js_link(self.server_nb, self.name, 
                                    confirm=False, 
                                    balise='ip_set_time_range_msg', 
                                    todel="%s#%s#%s"%(jour, plage['hdeb'], plage['hfin']),
                                    ip_set=self.group
                                    )
        libelle = ""
        title = "Supprimer la plage allant de %s au %s le %s"%(plage['hdeb'], plage['hfin'], jour)
        _class = "amon_ip_set_btn"
        return dict(href=href, icone=icone, libelle=libelle, title=title, _class=_class)

    def _get_copy_time_range_form(self):
        """ renvoie la description du formulaire de copie d'horaire """
        options = [{'libelle':name, 'name':name}for name in group_manager.get_all_groups(attr='name')if name!= self.group]
        ip_set_tocopy = {'name':'ip_set',
                           'libelle': "Copier les horaires d'un autre groupe", 
                          'options':[{'name':'groupes existants', 
                                        'options':options}]
                          }
        return dict(ip_set_tocopy=ip_set_tocopy, 
                    validate_time_range_copy=self._get_copy_time_range_btn()
                    )

    def _get_copy_time_range_btn(self):
        """ renvoie la description du bouton de validation pour la copie des plage temporelle """
        href = tools.make_form_link(self.server_nb, self.name, True, ['ip_set_time_range_copy', 'ip_set_current_set'], balise='ip_set_time_range_msg')
        title = "Copier les horaires de ce groupe"
        libelle = "Valider"
        icone = '/image/ok.gif'
        return dict(href=href, title=title, libelle=libelle, icone=icone)

    def set_time_range(self):
        """ définit une plage horaire """
        self.group = self.form_result['ip_set_current_set'][0]['value']
        resultat = tools.format_form_result(self.form_result['ip_set_time_range'])
        jours = resultat['day']
        hdeb = resultat['tstart']
        hfin = resultat['tstop']
        message = "Configuration de plage horaire:\\nLa restriction s'appliquera entre %s et %s\\n"%(hdeb, hfin)
        for jour in jours:
            if jour in time_manager.get_days_list():
                try:
                    time_manager.add_schedule(self.group, jour, hdeb, hfin)
                    message += "le %s\\n"%jour
                except Exception, mess:
                    message += "le %s:\\n     "%jour
                    message += mess.message
                    message += "\\n"
        
        # si le groupe est actif et les horaires sont actifs, la modif influe sur les règles iptables
        if self._get_group_datas()['active'] == '1' and self._get_group_datas()['time'] == '1':
            set_iptables_rules()

        return dict(message=message)

    def copy_time_range(self):
        """ copie les plages horaires d'un groupe à un autre """
        self.group = self.form_result['ip_set_current_set'][0]['value']
        resultat = tools.format_form_result(self.form_result['ip_set_time_range_copy'])
        group_to_copy_from = resultat['ip_set']
        time_manager.copy_schedules(group_to_copy_from, self.group)
        message = "Les plages horaires du groupe %s ont été associées au groupe %s.\\n"%(group_to_copy_from, self.group)
        return dict(message = message)

    def del_time_range(self):
        """ valide la suppression d'une plage horaire 
            on recoit:
            {'ip_set':nomdugroupe, 'todel':jour#heurededebut#heuredefin}
        """
        if self.params.has_key('ip_set'):
            self.group = self.params['ip_set'][0]
        else:
            raise Exception, "Erreur : il manque des données pour la suppression de plage horaire."
        
        plage = self.params['todel'][0].split('#')
        time_manager.del_schedule(self.group, plage[0], plage[1], plage[2])
        
        # si le groupe est actif et les horaires sont actifs, la modif influe sur les règles iptables
        if self._get_group_datas()['active'] == '1' and self._get_group_datas()['time'] == '1':
            set_iptables_rules()
        return dict(message = "La plage horaire du %s de %s à %s a été annulée."%(plage[0], plage[1], plage[2]))

