# -*- coding: utf-8 -*-
###########################################################################
# Eole NG - 2007  
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr 
#  
#  
# Gere les menus de l'interface
#       
###########################################################################
""" fonction de structuration des menus sur amon
    on peut ici configurer quelles actions sont présentées dans quelle rubrique

    par contre, le point d'entrée d'une rubrique est configuré dans les fichiers
    d'action (voir attribut category
"""

from twisted.python import log
from ead2.backend.actions import tools

menu_rules = 'regles', "p2p", 'horaire'
menu_postes = 'interdits', 'poste'
menu_user = 'banned_user', 'moderateur', 'white_user', 'grey_list'

def get_postes_menu(server_nb, page):
    return [pages[item]['item'](server_nb, page)for item in menu_postes]

def get_user_menu(server_nb, page):
    return [pages[item]['item'](server_nb, page)for item in menu_user]

def get_rules_menu(server_nb, page):
    return [pages[item]['item'](server_nb, page)for item in menu_rules]

def get_poste(server_nb,  page):
    item = {'href':tools.make_js_link(server_nb, 'navigation_poste'),
             'libelle':"Postes",
             'icone':'/image/amon/banned_poste.png',
             'title':pages['poste']['msg']}
    if page == 'poste':
        item['href'] = ''
    return item

def get_banned_user(server_nb,  page):
    item = {'href':tools.make_js_link(server_nb, 'navigation_banned_user'),
            'libelle':"Utilisateurs", 
            'icone':'/image/amon/banned_user.png',
            'title':pages['banned_user']['msg']}
    if page == 'banned_user':
        item['href'] = ''
    return item

def get_moderateur(server_nb,  page):
    item = {'href':tools.make_js_link(server_nb, 'navigation_moderateur'),
            'libelle':'Modérateurs', 
            'icone':'/image/amon/moderateur.png',
            'title':pages['moderateur']['msg']}
    if page == 'moderateur':
        item['href'] = ''
    return item

def get_white_user(server_nb, page):
    item = {'href':tools.make_js_link(server_nb, 'navigation_whitelist', current_page='white_user'),
             'libelle':'Liste blanche', 'icone':'/image/amon/white_user.png',
             'title':pages['white_user']['msg']}
    if page == 'white_user':
        item['href'] = ''
    return item

def get_grey_list(server_nb, page):
    item = {'href':tools.make_js_link(server_nb, 'navigation_whitelist', current_page='grey_list'),
            'libelle':'Sites de la liste blanche', 'icone':'/image/amon/grey_sites.png',
            'title':pages['grey_list']['msg']}
    if page == 'grey_list':
        item['href'] = ''
    return item

def get_regles(server_nb,  page):
    item = {'href':tools.make_js_link(server_nb, 'regles'),
             'libelle':"Regles de Pare-Feu", 'icone':'/image/amon/regles.png',
             'title':pages['regles']['msg']}
    if page == 'regles':
        item['href'] = ''
    return item

def get_interdits(server_nb,  page):
    item = {'href':tools.make_js_link(server_nb, 'destination_interdite'),
             'libelle':"Destinations interdites", 'icone':'/image/amon/noaccess.png',
              'title':pages['interdits']['msg']}
    if page == 'interdits':
        item['href'] = ''
    return item

def get_p2p(server_nb,  page):
    item = {'href':tools.make_js_link(server_nb, 'peertopeer'),
             'libelle':"Peer to Peer", 'icone':'/image/amon/p2p.png',
              'title':pages['p2p']['msg']}
    if page == 'p2p':
        item['href'] = ''
    return item

def get_horaire(server_nb,  page):
    item = {'href':tools.make_js_link(server_nb, 'horaire'),
             'libelle':"Horaires du pare-feu", 'icone':'/image/amon/hour.png',
              'title':pages['horaire']['msg']}
    if page == 'horaire':
        item['href'] = ''
    return item

def _get_menu(server_nb,  page):
    return pages[page]['menu'](server_nb,  page)

pages = {'poste':{'msg':"Gestion des postes à interdire",
                  'menu':get_postes_menu,
                  'item':get_poste},
             'banned_user':{'msg':"Gestion des utilisateurs à interdire",
                            'menu':get_user_menu,
                            'item':get_banned_user},
             'moderateur':{'msg':"Gestion des modérateurs",
                            "menu":get_user_menu,
                            'item':get_moderateur},
             'regles':{'msg':"Gestion des règles du Pare-Feu",
                       'menu':get_rules_menu,
                       'item':get_regles},
             'interdits':{'msg':"Interdire des destinations",
                          'menu':get_postes_menu,
                          'item':get_interdits},
             "p2p":{    'msg':"Interdiction des échanges de fichiers (peer to peer)",
                        'menu':get_rules_menu,
                        'item':get_p2p},
             'horaire':{'msg':"Horaires d'ouverture du pare-feu",
                        'menu':get_rules_menu,
                        'item':get_horaire},
             'grey_list':{'msg':"Ajout de site pour le mode liste blanche",
                            'menu':get_user_menu,
                            'item':get_grey_list},
             'white_user':{'msg':"Utilisateurs en liste blanche",
                            'menu':get_user_menu,
                            'item':get_white_user}
             }

