# -*- coding: utf-8 -*-
###########################################################################
# Eole NG - 2007
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
#
# Action navigation_access
#
# Gere les interdiction d'acces de poste ou d'url ainsi que l'autorisation
# de moderateur
#
###########################################################################
""" Action de gestion des filtres web sur un amon """
from twisted.python import log
from ead2.backend.lib.action import * # import des actions et des types
from ead2.backend.actions import tools
from ead2.backend.actions.amon import menus
import ead2.backend.actions.amon.amon_tools as t #fonctions de gestion des utilisateurs, postes....
from ead2.backend.config.config import SCRIPT_DIR
from ead2.lib.libead import uni
import commands
from os.path import join



#nom des formulaires
form_name = {'poste':['list_add'], 'banned_user':['list_add'],
            'moderateur':['list_add']}

used_templates = ['main','form', 'formselect', 'accordion']


class NavigationBannedUser(Action):
    """ interface de gestion des acces internet"""
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name', 'role'])
    name = 'navigation_banned_user'
    libelle = 'Utilisateurs'
    description = "Interdire des utilisateurs"
    category = "Configuration"
    request = Dict(default={}, doc="arguments de la requete en cour cote frontend",
                    keys=['server', 'action'])
    form_result = Dict(default={}, doc="retour formulaire de la requete en cour coté frontend",
                        keys = ['list_suppr', 'list_add'])
    def execute(self):
        """ renvoit les données pour la mise en forme
        """
        params, self.server_nb = tools.get_request(self.request)
        self.page = 'banned_user'
        if self.form_result != {}:
            try:
                datas = self._valid_banned_user()
                datas['toexec'] = tools.make_js_link(self.server_nb, self.name, confirm=True)
            except Exception, mess:
                log.err("Erreur: navigation_access.py, %s"%mess)
                datas = dict(message=mess.message)
            self.form_result = {}
            return self.send_frag(datas, template='amon_navigation_banned_user', templates=used_templates)

        result = {'titre': self.description}
        result.update(self._get_banned_user())
        result['sstitre'] =  ''
        result['menus'] = menus._get_menu(self.server_nb, self.page)
        result['validate'] = {'href':tools.make_form_link(self.server_nb, self.name,
                                                          True,
                                                          form_name[self.page],
                                                          balise='return_msg_container'),
                              'icone':'/image/amon/ok.gif',
                              'libelle':'Valider',
                              'title':'Valider les changements'}
        result['validation'] = tools.make_key_form_link(self.server_nb, self.name,
                                                        False,
                                                        form_name[self.page],
                                                        balise='return_msg_container')
        return self.send_all(result, template='amon_navigation_banned_user', templates=used_templates)

    def _get_banned_user(self):
        """ renvoie la description du formulaire d'interdiction d'utilisateur """
        ban_name = {'name':'name', 'onblur':"testVoid('name', 'error_name')",
                'libelle':"Login de l'utilisateur à interdire"}
        users = list()
        for user in t.get_users()['banned_users']:
            users.append({'name':user, 'libelle':user})
        banned_users = {'entete':"Utilisateurs interdits"}
        if users != []:
            banned_users.update({
               'validate':{'href':tools.make_form_link(self.server_nb, self.name,
                               True, ['list_suppr'], balise='return_msg_container'),
                           'icone':'/image/amon/supprimer_big.gif',
                           'libelle':'Réautoriser',
                           'title':'Réautoriser les utilisateurs selectionnés'},
              'select':{'name':'to_del', 'options':[{'name':"utilisateurs interdits", "options":users}],
              'libelle':"Veuillez sélectionner le ou les utilisateurs à (ré)autoriser", 'multi':"ok"}})
        else:
            banned_users.update({"msg": "Aucun utilisateur n'est interdit"})
        return dict(ban_name=ban_name, listing=banned_users, focus='name')

    def _valid_banned_user(self):
        """ valide l'interdiction d'utilisateur """
        if not self.form_result.has_key('list_add') and not self.form_result.has_key('list_suppr'):
            raise Exception, "Erreur: il manque des données pour l'interdiction d'utilisateur."
        # on interdit des users
        if self.form_result.has_key('list_add'):
            result = self.form_result['list_add']
            user = tools.format_form_result(result)
            username = user['name']
            t.add_user(username, 'banned_user')
            if t.dansguardian_reload():
                return dict(message="L'utilisateur %s a bien été interdit, le filtre a bien été redémarré."%username)
            else:
                log.err("Erreur: navigation_access.py, erreur au lancement de dansguardian_reload()(1).")
                raise Exception, "Erreur: une erreur s'est produite au rechargement du filtre (dansguardian)."

        # réautorisation d'utilisateur interdits
        elif self.form_result.has_key('list_suppr'):
            result = self.form_result['list_suppr']
            users = tools.format_form_result(result)
            if not users.has_key('to_del'):
                log.err("Erreur: navigation_access.py, il manque des données pour la réautorisation d'utilisateur.")
                raise Exception, "Erreur: il manque des données pour la réautorisation d'utilisateur."
            if len(users['to_del']) > 1: message = "La réautorisation des utilisateurs:\\n"
            else: message = "La réautorisation de l'utilisateur: \\n"
            for user in users['to_del']:
                t.autorise_user(user)
                message += "%s\\n"%user
            message += "s'est bien déroulée.\\n"
            if t.dansguardian_reload():
                message += "Le filtre a bien redémarré."
                return dict(message=message)
            else:
                log.err("Erreur: navigation_access.py, erreur au lancement de dansguardian_reload()(2).")
                raise Exception, "Erreur: une erreur s'est produite au rechargement du filtre (dansguardian)."

class Moderateur(Action):
    """ interface de gestion des modérateurs """
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name', 'role'])
    name = 'navigation_moderateur'
    libelle = 'Modérateur'
    description = 'Ajouter des modérateurs'
    category = None
    request = Dict(default={}, doc="arguments de la requete en cour cote frontend",
                                keys=['server', 'action'])
    form_result = Dict(default={}, doc="retour formulaire de la requete en cour coté frontend",
                        keys = ['list_suppr', 'list_add'])
    def execute(self):
        """ renvoit les données pour la mise en forme
        """
        params, self.server_nb = tools.get_request(self.request)
        self.page = 'moderateur'
        if self.form_result != {}:
            try:
                datas = self._valid_moderateur()
                datas['toexec'] = tools.make_js_link(self.server_nb, self.name, confirm=True)
            except Exception, mess:
                log.err("Erreur: navigation_access.py, %s"%mess)
                datas = dict(message=mess.message)
            self.form_result = {}
            return self.send_frag(datas, template='amon_navigation_moderateur', templates=used_templates)

        result = {'titre': self.description}
        result.update(self._get_moderateur())
        result['sstitre'] =  ''
        result['menus'] = menus._get_menu(self.server_nb, self.page)
        result['validate'] = {'href':tools.make_form_link(self.server_nb, self.name,
                                                            True,
                                                            form_name[self.page],
                                                            balise='return_msg_container'),
                                                            'icone':'/image/amon/ok.gif',
                                                            'libelle':'Valider',
                                                            'title':'Valider les changements'}
        result['validation'] = tools.make_key_form_link(self.server_nb, self.name,
                                                            False,
                                                            form_name[self.page],
                                                            balise='return_msg_container')
        return self.send_all(result, template='amon_navigation_moderateur', templates=used_templates)

    def _get_moderateur(self):
        """ renvoie la description du formulaire d'inscription de modérateur """
        moderateur_name = {'name':'name', 'onblur':"testVoid('name', 'error_name')",
                'libelle':"Login du modérateur à ajouter"}
        moderateurs = []
        for moderateur in t.get_users()['moderateurs']:
            moderateurs.append({'name':moderateur, 'libelle':moderateur})
        moderateur_users =  {'entete':"Supprimer un modérateur"}
        if moderateurs != []:
            moderateur_users.update({
                'validate':{'href':tools.make_form_link(self.server_nb, self.name,
                                                        True,
                                                        ['list_suppr'],
                                                        balise='return_msg_container'),
                            'icone':'/image/amon/supprimer_big.gif',
                            'libelle':'Supprimer',
                            'title':'Supprimer les modérateurs selectionnés'},
                'select':{'name':'to_del',
                            'options':[{'name':"moderateurs", "options":moderateurs}],
                            'libelle':"Veuillez sélectionner le ou les modérateurs à supprimer",
                            'multi':"ok"}})
        else:
            moderateur_users.update({"msg": "Aucun modérateur n'a été défini"})
        return dict(moderateur_name=moderateur_name, listing=moderateur_users, focus='name')

    def _valid_moderateur(self):
        """ valide l'ajout du moderateur """
        if not self.form_result.has_key('list_add') and not self.form_result.has_key('list_suppr'):
            log.err("Erreur: navigation_access.py, il manque des données pour l'ajout de modérateur(formulaire).")
            raise Exception, "Erreur: il manque des données pour la gestion des modérateurs."

        # on ajoute un modérateur
        if self.form_result.has_key('list_add'):
            result = self.form_result['list_add']
            user = tools.format_form_result(result)
            if not user.has_key('name'):
                log.err("Erreur: navigation_access.py, il manque des données pour l'ajout de modérateur(valeurs).")
                raise Exception, "Erreur: il manque des donnés pour l'ajout de modérateur."
            username = user['name']
            t.add_user(username, 'moderateur')
            if t.dansguardian_reload():
                return dict(message="L'utilisateur %s a bien été inscrit comme modérateur, le filtre a bien été rechargé."%username)
            else:
                log.err("Erreur: navigation_access.py, erreur au lancement de dansguardian_reload()(3).")
                raise Exception, "Erreur: une erreur s'est produite au rechargement du filtre (dansguardian)."

        # suppression de modérateurs
        elif self.form_result.has_key('list_suppr'):
            result = self.form_result['list_suppr']
            users = tools.format_form_result(result)
            if not users.has_key('to_del'):
                log.err("Erreur: navigation_access.py, il manque des données pour la suppression de modérateurs.")
                raise Exception, "Erreur: il manque des donnés pour la suppression de modérateur."
            if len(users['to_del']) > 1: message = "La suppression de la liste de modérateurs des utilisateurs:\\n"
            else: message = "La suppression de la liste de modérateurs de l'utilisateur: \\n"
            for user in users['to_del']:
                t.autorise_user(user)
                message += "%s\\n"%user
            message += "s'est bien déroulée.\\n"
            if t.dansguardian_reload():
                message += "Le filtre a bien redémarré."
                return dict(message=message)
            else:
                log.err("Erreur: navigation_access.py, erreur au lancement de dansguardian_reload()(4).")
                raise Exception, "Erreur: une erreur s'est produite au rechargement du filtre (dansguardian)."

class Poste(Action):
    """ interface de gestion des acces internet"""
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name', 'role'])
    name = 'navigation_poste'
    libelle = 'Interdiction de poste'
    description = 'Interdire des postes'
    category = None #"Configuration"
    request = Dict(default={}, doc="arguments de la requete en cour cote frontend",
                    keys=['server', 'action'])
    form_result = Dict(default={}, doc="retour formulaire de la requete en cour coté frontend",
                        keys = ['list_suppr', 'list_add', 'all_list_suppr'])
    def execute(self):
        """ renvoit les données pour la mise en forme
        """
        params, self.server_nb = tools.get_request(self.request)
        self.page = 'poste'
        result = {'titre': self.description}
        if self.form_result != {}:
            try:
                datas = self._valid_poste()
                datas['toexec'] = tools.make_js_link(self.server_nb, self.name, confirm=True)
            except Exception, mess:
                log.err("Erreur: navigation_access.py, %s"%mess)
                datas = dict(message=mess.message)
            self.form_result = {}
            return self.send_frag(datas, template='amon_navigation_poste', templates=used_templates)
        ## 1 -
        result.update(self._get_poste())
        result['sstitre'] =  ''
        result['menus'] = menus._get_menu(self.server_nb, self.page)
        result['validate'] = {'href':tools.make_form_link(self.server_nb, self.name,
                                                            True,
                                                            form_name[self.page],
                                                            balise='return_msg_container'),
                                                            'icone':'/image/amon/ok.gif',
                                                            'libelle':'Valider',
                                                            'title':'Valider les changements'}
        result['validation'] = tools.make_key_form_link(self.server_nb, self.name,
                                                        False,
                                                        form_name[self.page],
                                                        balise='return_msg_container')
        return self.send_all(result, template='amon_navigation_poste', templates=used_templates)

    def _get_poste(self):
        """ decrit le formulaire a presenter pour la gestion des postes """
        ip = {'name':'ip', 'onblur':"testVoid('ip', 'error_ip')",
                    'libelle':"adresse / sous réseau à interdire"}
        heure_deb = {'name':'heure_deb', 'options':[{'name':'heure de début', 'options':t.get_hours('7:00')}],
                        'libelle':"Heure de début"}
        heure_fin = {'name':'heure_fin', 'options':[{'name':'heure de fin', 'options':t.get_hours('19:00')}],
                    'libelle':"Heure de fin"}
        jour_deb = {'name':'jour_deb', 'options':[{'name':'jour de début', 'options':t.get_days('lundi')}],
                        'libelle':" du "}
        jour_fin = {'name':'jour_fin', 'options':[{'name':'jour de fin', 'options':t.get_days('dimanche')}],
                        'libelle':" au "}
        niveau = {'options':[("Toute activité réseau",{'name':'niveau', 'value':'all', 'inline':'ok'}),
            ('Seulement le web',{'name':'niveau', 'value':'web', 'checked':'true', 'inline':'ok'})],
                'libelle':'Niveau de restriction'}
        #postes interdits de web: description de la section de reautorisation
        # une entete de tableau, une selectbox, un bouton valider
        postes = []
        for poste in t.get_forbidden_postes():
            poste = poste.split("#")
            libelle = "%s de %s à %s du %s au %s"%(poste[0],poste[1], poste[2], poste[3], poste[4])
            nom = poste[0]
            postes.append({'name':nom, 'libelle':libelle})
        if postes != []:
            forbidden_poste = {'entete':"Postes interdits d'accès web",
                 'validate':{'href':tools.make_form_link(self.server_nb, self.name,
                                True, ['list_suppr'], balise='return_msg_container'),
                             'icone':'/image/amon/supprimer_big.gif',
                             'libelle':'Réautoriser',
                             'title':'Réautoriser les postes selectionnés'},
                 'select':{'name':'to_del', 'options':[{'name':"ips/sous-réseau interdits", "options":postes}],
                     'libelle':"Veuillez sélectionner le ou les postes à (ré)autoriser",'multi':'ok'}
                                }
        else:
            forbidden_poste = {"msg": "Aucun poste n'est interdit d'accès web", 'entete':"Postes interdits d'accès web"}
        # postes interdits sur tout le reseau: description de la section de reautorisation
        # une entete de tableau, une selectbox, un bouton valider
        postes = []
        for poste in t.get_all_forbidden_postes():
            poste = poste.split("#")
            libelle = "%s est interdit tout le temps"%poste[0]
            nom = poste[0]
            postes.append({'name':nom, 'libelle':libelle})
        if postes != []:
            all_forbidden_poste = {'entete':"Postes interdits sur tout le réseau (ne tient pas compte des horaires)",
                 'validate':{'href':tools.make_form_link(self.server_nb, self.name,
                               True, ['all_list_suppr'], balise='return_msg_container'),
                             'icone':'/image/amon/supprimer_big.gif',
                             'libelle':'Réautoriser',
                             'title':'Réautoriser les postes selectionnés'},
                 'select':{'name':'to_del', 'options':[{'name':"ips/sous-réseau interdits", "options":postes}],
                     'libelle':"Veuillez sélectionner le ou les postes à (ré)autoriser",'multi':'ok'}
                                }
        else:
            all_forbidden_poste = {"msg": "Aucun poste n'est interdit totalement",
                        'entete':"Postes interdits sur tout le réseau (ne tient pas compte des horaires)"}
        return dict(ip=ip,
                    heure_deb=heure_deb,
                    heure_fin=heure_fin,
                    jour_deb=jour_deb,
                    jour_fin=jour_fin,
                    niveau=niveau,
                    listing=forbidden_poste,
                    all_forbidden_poste=all_forbidden_poste,
                    focus='ip')

    def _valid_poste(self):
        """ valide l'interdiction de poste """
        if not self.form_result.has_key('list_add') and not self.form_result.has_key('list_suppr') and not self.form_result.has_key('all_list_suppr'):
            log.err("Erreur: navigation_access.py, il manque des données pour l'interdiction de poste(formulaire).")
            raise Exception, "Erreur: il manque des données pour l'interdiction de poste."

        # on interdit des postes
        if self.form_result.has_key('list_add'):
            result = self.form_result['list_add']
            poste = tools.format_form_result(result)
            for key in ['ip', 'heure_deb', 'heure_fin', 'jour_deb', 'jour_fin', 'niveau']:
                if key not in poste.keys():
                    log.err("Erreur: navigation_access.py, il manque des données pour l'interdiction de poste(valeurs).")
                    raise Exception, "Erreur: il manque des données pour l'interdiction de poste."
            ip=poste['ip']
            deb=poste['heure_deb']
            fin=poste['heure_fin']
            jdeb=poste['jour_deb']
            jfin=poste['jour_fin']
            niveau=poste['niveau']
            if niveau == 'web':
                t.forbid_poste(ip, deb, fin, jdeb, jfin)
                message = "Le poste %s est interdit d'accès web du %s au %s entre %s et %s.\\n"%(ip, jdeb, jfin, deb, fin)
            else:
                t.all_forbid_poste(ip, deb, fin, jdeb, jfin)
                message = "Le poste %s est interdit d'accès réseau du %s au %s entre %s et %s.\\n"%(ip, jdeb, jfin, deb, fin)
            if t.bastion_restart():
                message += "Le pare-feu a été relancé."
                return dict(message=message)
            else:
                log.err("Erreur: navigation_access.py, erreur au lancement de 'service bastion restart'(1).")
                raise Exception, "Erreur: une erreur s'est produite au lancement du pare-feu."

        # on leve des interdictions de postes
        elif self.form_result.has_key('list_suppr'):
            result = self.form_result['list_suppr']
            postes = tools.format_form_result(result)
            if not postes.has_key('to_del'):
                log.err("Erreur: navigation_access.py, il manque des données pour l'autorisation de poste(1).")
                raise Exception, "Erreur: il manque des données pour l'autorisation de poste."
            t.autorize_postes(postes['to_del'])
            if t.bastion_restart():
                return dict(message="Les postes ont été réautorisés, le pare-feu a été relancé.")
            else:
                log.err("Erreur: navigation_access.py, erreur au lancement de 'service bastion restart'(2).")
                raise Exception, "Erreur: une erreur s'est produite au lancement du pare-feu."

        elif self.form_result.has_key('all_list_suppr'):
            result = self.form_result['all_list_suppr']
            postes = tools.format_form_result(result)
            if not postes.has_key('to_del'):
                log.err("Erreur: navigation_access.py, il manque des données pour l'autorisation de poste(2).")
                raise Exception, "Erreur: il manque des données pour l'autorisation de poste."
            t.autorize_all_postes(postes['to_del'])
            if t.bastion_restart():
                return dict(message="Les postes ont été réautorisés, le pare-feu a été relancé.")
            else:
                log.err("Erreur: navigation_access.py, erreur au lancement de 'service bastion restart'(3)")
                raise Exception, "Erreur: une erreur s'est produite au lancement du pare-feu."


