# -*- coding: utf-8 -*-
###########################################################################
# Eole NG - 2007  
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr 
#  
# Action navigation_filters
#  
# Edite les differentes regles de filtrage
#       
###########################################################################

""" Action de gestion des filtres web sur un amon """

from twisted.python import log
from ead2.backend.lib.action import * # import des actions et des types
from ead2.backend.actions.tools import get_request, make_form_link, make_confirm_link, make_js_link, make_long_js_link, make_key_form_link
import ead2.backend.actions.amon.amon_tools as t
from ead2.backend.actions.amon.config_dans import active_c, active_m, desactive
from ead2.backend.config.config import SCRIPT_DIR, CONFIG_DIR
from ead2.backend.config.filenames import navigation_filters, dg_reload
from ead2.lib.libead import uni
import commands, os
from os.path import isfile, join
from ead2.lib.error import MissingKey, MissingValue

# message de la page
page_msgs = {'filtres':"Activation des filtres facultatifs", 
             'filtrage':"Règles de filtrage pour la navigation",
             'blacklist':"Gestion des sites interdits",
             'whitelist':"Gestion des sites autorisés",
             "extensions":"Interdictions d'extension", 
             "mime":"Interdiction de type mime",
             "signalement": "Signalement de sites et d'URL"
             }
# id html des formulaires à valider
form_name = {'filtres':['filtres'], 
             'filtrage':['dans', 'current_page'], 
             'blacklist':['current_page', 'list_add'], 
             'whitelist':['current_page', 'list_add'], 
             'extensions':['current_page', 'list_add'],
             'mime':['current_page', 'list_add']}


filtrage_dans_status = navigation_filters['dans_status'] #'/var/www/ead/tmp/filtrage-contenu'
blacklist_filename =  navigation_filters['blacklist']      #'/var/lib/blacklists/local/domains'
whitelist_filename = navigation_filters['whitelist']       #'/var/lib/blacklists/local/white'
extensions_filename = navigation_filters['extensions']     #'/var/lib/blacklists/local/extensions'
mime_filename = navigation_filters['mime']                #'/var/lib/blacklists/local/types_mime'
list_mime_filename = navigation_filters['list_mime']      #join(CONFIG_DIR, 'mime_liste.txt')
signalement_msg = navigation_filters['signalement_msg']    #join(CONFIG_DIR, 'signalement_msg.txt')

# templates utilisés
used_templates = ['main', 'form', 'formselect', 'accordion']

def parse_mimes():
    """ parse les types mimes et renvoie 2 listes de dico 
        {'name':nom de l'option, 'libelle':libelle de l'option'}
        les types disponibles et ceux déjà interdits
    """
    all = t.get_lines(list_mime_filename)
    forbiddens = t.get_lines(mime_filename)
    
    available_mime = []
    forbidden_mime = []

    for mime in all:
        mime = mime.split(';')
        libelle = mime[0].strip()
        if len(mime)>=3:
            if mime[2].strip():
                libelle += " (%s)"%mime[2]
        name = mime[1].strip()
        if not name in forbiddens:
            available_mime.append({'name':name, 'libelle':libelle})
        else:
            forbidden_mime.append({'name':name, 'libelle':libelle})
    return available_mime, forbidden_mime


class NavigationFilters(Action):
    """ interface de gestion des filtres de sites web"""
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name', 'role'])
    name = 'navigation_filters'
    libelle = 'Sites'
    description = 'Gérer les filtres internet'
    category = "Configuration"
    request = Dict(default={}, doc="arguments de la requete en cour cote frontend", 
                    keys=['server', 'action', 'current_page'])
    form_result = Dict(default={}, doc="retour formulaire de la requete en cour coté frontend",
                        keys = ['filtres', 'dans', 'current_page', 'list_add', 'list_suppr'
                                ])
    def execute(self):
        """ renvoie les données pour la mise en forme
            1 - récupère la page en cour de traitement
            2 - renvoie la description du formulaire et le menu
            3 - valide le retour formulaire
        """
        params, self.server_nb = get_request(self.request)
        self.page = 'filtres'
        if self.form_result != {}:
            ## 1 -
            if self.form_result.has_key('current_page'):
                self.page = self.form_result['current_page'][0]['value']
            ## 3 -
            try:
                datas = self._valid_form()
                datas['toexec'] = tools.make_js_link(self.server_nb, self.name, current_page=self.page)
            except (MissingKey, MissingValue, Exception), mess:
                datas = {'message':mess.message}
            self.form_result = {}
            return self.send_frag(datas, template='amon_navigation_filters', templates=used_templates)
        ## 2 -
        if params.has_key('current_page'):
            self.page = params['current_page'][0]
        ## 1 -
        result = {'titre': "Gestion des filtres internet"}
        result['menus'] = self._get_menus()
        result['sstitre'] =  page_msgs[self.page]
        result['menus'] = self._get_menus()
        result.update(self._get_form())
        if form_name.has_key(self.page):
            result['validate'] = {'href':make_form_link(self.server_nb, self.name,
                                                        True, 
                                                        form_name[self.page], 
                                                        balise='return_msg_container'),
                                  'icone':'/image/amon/ok.gif',
                                  'libelle':'Valider',
                                  'title':'Valider les changements'}
            result['validation'] = make_key_form_link(self.server_nb, self.name, 
                                                        False, 
                                                        form_name[self.page], 
                                                        balise='return_msg_container')
        result['page'] = {'name':'current_page', 'default_value':self.page}
        return self.send_all(result, template='amon_navigation_filters', templates=used_templates)

    def _get_form(self):
        """ renvoie la description du formulaire selon le nom de page """
        if self.page == 'filtres':
            return self._filtres_form()
        elif self.page == 'filtrage':
            return self._filtrage_form()
        elif self.page == 'whitelist':
            return self._whitelist_form()
        elif self.page == 'blacklist':
            return self._blacklist_form()
        elif self.page == 'extensions':
            return self._extensions_form()
        elif self.page == 'mime':
            return self._mime_form()
        elif self.page == 'signalement':
            return self._signalement_form()

    def _filtres_form(self):
        """ renvoie la description du formulaire de gestion des filtres optionnels """
        options = t.get_filtres_options()
        optionnal_filters = []
        if not options:
            return dict()
        else:
            users_opt = t.get_filtres_user_options()
            for option in options:
                description = option['libelle']
                radio_actif = {'name':option['name'], 'value':'actif'}
                radio_inactif = {'name':option['name'], 'value':'inactif'}
                actif = False
                for user_opt in users_opt:
                    if option['name'] in user_opt:
                        actif = True
                if actif: radio_actif['checked'] = 'ok'
                else:radio_inactif['checked'] = 'ok'
                optionnal_filters.append((description, radio_actif, radio_inactif))
            all_active_btn = {'icone':'', 'title':'', 'libelle':'tous', 'href':"javascript:formCheckActif('filtres')"}
            all_unactive_btn = {'icone':'', 'title':'', 'libelle':'tous', 'href':"javascript:formCheckInactif('filtres')"}
            return dict(optionnal_filters=optionnal_filters, all_active_btn=all_active_btn,all_unactive_btn=all_unactive_btn)

    def _filtrage_form(self):
        """ renvoie la description du formulaire de gestion du filtrage syntaxique """
        desactive = {'name':'dans', 'value':'off'}
        meta = {'name':'dans', 'value':'meta'}    
        active = {'name':'dans', 'value':'on'}    
        if isfile(filtrage_dans_status):
            lines = file(filtrage_dans_status, 'r').read().splitlines()
            if '0' in lines[0]:
                desactive['checked'] = 'ok'
            elif '1' in lines[0]:
                meta['checked'] = 'ok'
            else:
                active['checked'] = 'ok'
        else:
            active['checked'] = 'ok'
        desactive = ('Désactivé', desactive)
        meta = ("Sur les entêtes de pages WEB (recommandé)",meta)
        active = ('Sur la totalité de la page', active)
        return dict(dans_unactive=desactive, dans_meta=meta, dans_active=active)
   
    def _blacklist_form(self):
        """ renvoie la description du formulaire de gestion des listes de sites interdits """
        blacklist_site = {'name':'addsite', 'libelle': "Veuillez entrer l'adresse d'un site à interdire", 
                        'onblur':"testVoid('addsite', 'error_addsite')"}
        blacks = t.get_lines(blacklist_filename)
        blacks = [{'name':black, 'libelle':black}for black in blacks]
        blacklist = {'entete':"Sites interdits"}
        if blacks != []:
            blacklist.update({'validate':{'href':make_form_link(self.server_nb, self.name,
                                                                True, 
                                                                ['current_page', 'list_suppr'], 
                                                                balise='return_msg_container'),
                                          'icone':'/image/amon/supprimer_big.gif',
                                          'libelle':'Réautoriser',
                                          'title':'Réautoriser les sites selectionnés'},
                              'select':{'name':'list_suppr', 
                                        'options':[{'name':"sites interdits", "options":blacks}],
                                        'libelle':"Selectionner le ou les site(s) à réautoriser", 
                                        'multi':'ok'}
                                })
        else:
            blacklist.update({'msg':"Aucun site n'est interdit"})
        return dict(blacklist_site=blacklist_site, listing=blacklist, focus='addsite')

    def _whitelist_form(self):
        """ renvoie la description du formulaire de gestion des listes de sites autorises """
        whitelist_site = {'name':'addsite', 'libelle': "Veuillez entrer l'adresse d'un site à (ré)autoriser",
                        'onblur':"testVoid('addsite', 'error_addsite')"}
        whites = t.get_lines(whitelist_filename)
        whites = [{'name':white, 'libelle':white}for white in whites]
        whitelist = {'entete':"Sites autorisés"}
        if whites != []:
            whitelist.update({'validate':{'href':make_form_link(self.server_nb, self.name,
                                                               True, 
                                                               ['current_page', 'list_suppr'],
                                                               balise='return_msg_container'), 
                                          'icone':'/image/amon/supprimer_big.gif',
                                          'libelle':'Réinterdire', 
                                          'title':'Réinterdire les sites selectionnés'},
                              'select':{'name':'list_suppr', 
                                        'options':[{'name':"sites autorisés", "options":whites}],
                                        'libelle':"Selectionner le ou les site(s) à interdire", 
                                        'multi':'ok'}
                              })
        else:
            whitelist.update({'msg':"Aucun site n'a été réautorisé"})
        return dict(whitelist_site=whitelist_site, listing=whitelist, focus='addsite')

    def _extensions_form(self):
        """ renvoie la description du formulaire de gestion des extensions """
        extension_site = {'name':'addsite', 'libelle': "Veuillez entrer l'extension à interdire",
                            'onblur':"testVoid('addsite', 'error_addsite')"}
        exts = t.get_lines(extensions_filename)
        exts = [{'name':ext, 'libelle':ext}for ext in exts]
        extensions = {'entete':"Extensions interdites"}
        if exts != []:
            extensions.update({'validate':{'href':make_form_link(self.server_nb, self.name,
                                                   True, ['current_page', 'list_suppr'],
                                                   balise='return_msg_container'), 
                                           'icone':'/image/amon/supprimer_big.gif',
                                           'libelle':'Réautoriser', 
                                           'title':'Réautoriser les extensions selectionnées'},
                               'select':{'name':'list_suppr', 
                                         'options':[{'name':"extensions interdites", "options":exts}],
                                         'libelle':"Selectionner la ou les extension(s) à réautoriser", 
                                         'multi':'ok'}
                               })
        else:
            extensions.update({'msg':"Aucune extension n'est interdite"})
        return dict(extension_site=extension_site, listing=extensions, focus='addsite')

    def _mime_form(self):
        """ renvoie la description du formulaire de gestion des types mime """
        mimetypes, mimes = parse_mimes()
        mime_name = {'name':'addsite', 'libelle': "Veuillez choisir le type MIME à interdire", 
                'options':[{'name':'types MIME', 'options':mimetypes}], 'id':'addsite'}
        typesmime = {'entete':"Types MIME interdits"}
        if mimes != []:
            typesmime.update({'validate':{'href':make_form_link(self.server_nb, self.name,
                                                                True, 
                                                                ['current_page','list_suppr'], 
                                                                balise='return_msg_container'), 
                                          'icone':'/image/amon/supprimer_big.gif',
                                          'libelle':'Réautoriser', 
                                          'title':'Réautoriser les types MIME selectionnés'},
                              'select':{'name':'list_suppr', 
                                        'options':[{'name':"types MIME interdits", "options":mimes}],
                                        'libelle':"Selectionner le ou les type(s) MIME à réautoriser", 
                                        'multi':'ok'}
                              })
        else:
            typesmime.update({'msg':"Aucun type MIME n'est interdit"})
        return dict(mime_name=mime_name, listing=typesmime)

    def _signalement_form(self):
        """ renvoie les données pour la page de signalement """
        guide = {'href':"javascript:OuvreFenetre('http://www.educnet.education.fr/aiedu/guide1.htm', 'Educnet')",
                         'libelle':"Plus d'information", 
                         'icone':"/image/back.png", 'title':"Plus d'information"}
        form_link = {'href':"javascript:OuvreFenetre('http://aiedu.education.fr/', 'Educnet');", 
                     "libelle":"Accès direct au formulaire de déclaration de site",
                     "icone":"/image/back.png", 
                     'title':'formulaire de déclaration de site'}
        msg = t.get_lines(signalement_msg)
        return dict(signalement_text=msg, signalement_form_link=form_link, signalement_guide=guide)

    def _valid_form(self):
        """ gere la validation des formulaires """
        if self.page == 'filtres':
            return self._valid_filtres()
        elif self.page == 'filtrage':
            return self._valid_filtrage()
        elif self.page == 'whitelist':
            return self._valid(whitelist_filename, 'white')
        elif self.page == 'blacklist':
            return self._valid(blacklist_filename, 'black')
        elif self.page == 'extensions':
            return self._valid(extensions_filename, 'extension')
        elif self.page == 'mime':
            return self._valid(mime_filename)
        elif self.page == 'signalement':
            pass

    def _valid_filtres(self):
        """ valide les activations de filtres optionnels """
        result = self.form_result['filtres']
        # on recupere les valeurs checkees et actifs
        filtres = []
        for filtre in result:
            if not filtre['checked'] == 'false':
                if filtre['value'] == 'actif':
                    filtres.append(filtre)
        t.write_filtres(filtres)
        t.ajout_filtre()
        t.dansguardian_reload()
        return dict(message="Les filtres optionnels ont bien été modifiés")

    def _valid_filtrage(self):
        """ valide le formulaire pour les filtrages syntaxiques """
        dans = self.form_result['dans']
        dans_opt = None
        for opt in dans:
            if opt['checked'] == 'true':
                dans_opt = opt['value']
        if dans_opt == 'off':
            a = file(filtrage_dans_status, 'w')
            a.write('0')
            a.close()
            desactive()
        elif dans_opt == 'meta':
            a = file(filtrage_dans_status, 'w')
            a.write('1')
            a.close()
            active_m()
        elif dans_opt == 'on':
            a = file(filtrage_dans_status, 'w')
            a.write('2')
            a.close()
            active_c()
        t.dansguardian_reload()
        return dict(message="Les règles de filtrage syntaxique ont bien été modifiés")

    def _valid(self, filename, data_type=''):
        """ valide les ajouts suppressions de parametres pour les listes, extensions et type mime"""
        if self.form_result.has_key('list_add'):
            result = tools.format_form_result(self.form_result['list_add'])

            add = result.values()[0].strip()
            log.msg(add)
            # traitement des données entrées
            if data_type == 'black':
                whitelist = t.get_lines(whitelist_filename)
                if add in whitelist:
                    raise MissingValue, "Attention : Veuillez supprimer ce site de la liste des sites \\n autorisées avant de l'interdire"
                t.add_url(filename, add)
                message = "L'url %s a bien été ajoutée à la liste des sites interdits."%add
            elif data_type == 'white':
                blacklist = t.get_lines(blacklist_filename)
                if add in blacklist:
                    raise MissingValue, "Attention : Veuillez supprimer ce site de la liste des sites \\n interdits avant de le réautoriser"
                t.add_url(filename, add)
                message = "L'url %s a bien été ajoutée à la liste des sites autorisés."%add
            elif data_type == 'extension':
                t.add_extension(filename, add)
                message = "L'extension %s a bien été ajoutée à la liste des extensions interdites."%add
            else:
                # pas de traitement particulier
                t.add_lines(filename, add)
                message = "Le type mime %s a bien été ajouté à la liste des types mime interdits."%add
        elif self.form_result.has_key('list_suppr'):
            result = tools.format_form_result(self.form_result['list_suppr'])
            supprs = result.values()[0]
            t.del_lines(filename, supprs)
            if data_type == 'black': 
                message = "Les sites %s ont bien été enlevés de la liste des sites interdits."
            elif data_type == 'white':
                message = "Les sites %s ont bien été enlevés de la liste des sites autorisés."
            elif data_type == 'extension':
                message = "Les extensions %s ont bien été réautorisées."
            else:
                message = "Les types mime %s ont bien été réautorisés."
            supprs_msg = '\\n'
            for i in supprs: supprs_msg += "%s\\n"%i
            message = message%supprs_msg
        else:
            raise MissingValue, "Erreur : il manque des paramètres."
        t.dansguardian_reload()
        return dict(message=message)
            
    def _get_menus(self):
        """renvoie la description du menu de la section filtrage """
        item1 = {'href':make_js_link(self.server_nb, self.name, current_page='filtres'),
                 'libelle':"Filtres", 
                 'icone':'/image/amon/filters.png', 
                 'title':page_msgs['filtres']}
        item2 = {'href':make_js_link(self.server_nb, self.name, current_page='filtrage'),
                 'libelle':"Filtrage", 
                 'icone':'/image/amon/filters.png',
                 'title':page_msgs['filtrage']}
        item3 = {'href':make_js_link(self.server_nb, self.name, current_page='blacklist'),
                 'libelle':'Sites interdits', 
                 'icone':'/image/amon/filters.png',
                 'title':page_msgs['blacklist']}
        item4 = {'href':make_js_link(self.server_nb, self.name, current_page='whitelist'),
                 'libelle':'Sites autorisés', 
                 'icone':'/image/amon/filters.png',
                 'title':page_msgs['whitelist']}
        item5 = {'href':make_js_link(self.server_nb, self.name, current_page='extensions'),
                 'libelle':'Extensions', 
                 'icone':'/image/amon/filters.png',
                 'title':page_msgs['extensions']}
        item6 = {'href':make_js_link(self.server_nb, self.name, current_page='mime'),
                 'libelle':'Type MIME', 
                 'icone':'/image/amon/filters.png',
                 'title':page_msgs['mime']}
        item7 = {'href':make_js_link(self.server_nb, self.name, current_page='signalement'),
                 'libelle':'Signalement', 
                 'icone':'/image/amon/filters.png',
                 'title':page_msgs['signalement']}
        if self.page == 'filtres':
            item1['href'] = ''
        elif self.page == 'filtrage':
            item2['href'] =''
        elif self.page == 'blacklist':
            item3['href'] = ''
        elif self.page == 'whitelist':
            item4['href'] = ''
        elif self.page == 'extensions':
            item5['href'] = ''
        elif self.page == 'mime':
            item6['href'] = ''
        else:
            item7['href'] = ''
        return [item1, item2, item3, item4, item5, item6, item7]
