# -*- coding: utf-8 -*-
###########################################################################
# Eole NG - 2007  
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr 
#  
# Action navigation_visit
#  
# Permet la consultation des logs par utilisateur, poste , horaire, url ou
# accès refusé
#       
###########################################################################

""" Action de consultations des navigations web sur un amon """

from twisted.python import log
from ead2.backend.lib.action import * # import des actions et des types
from ead2.backend.actions import tools #import get_request, format_form_result, make_form_link, make_js_link
from ead2.backend.actions.amon.amon_tools import get_return_btn
from ead2.lib.libead import uni
import os, xmlrpclib, urllib
from os.path import isfile
from datetime import datetime
from ead2.backend.config.filenames import navigation_visit
from ead2.backend.actions.lib.logparser import file_listing, logparsing
from ead2.lib.error import MissingKey, MissingValue

#messages
mode_msgs = {'hour':'par tranches horaires', 'user':'par utilisateur', 'client':'par poste'}
# fichier
dansguardian_log = navigation_visit['dansguardian_log']
# template additionnel
used_templates = ['main', 'form', 'formselect', 'accordion']

def get_log_days():
    """ renvoie un dico avec les logs disponible et le jour correspondant """
    available = collect.available_days_in_logs()
    result = []
    semaine = []
    for a,b in available['week']:
        semaine.append({'libelle':a, 'name':b})
    result.append({'name':'Cette semaine', 'options':semaine})
    if available.has_key('lastweek'):
        precsemaine=[]
        for a,b in available['lastweek']:
            precsemaine.append({'libelle':a, 'name':b})
        result.append({'name':'La semaine dernière', 'options':precsemaine})
    return result

def get_days(today=False):
    """ renvoie la liste des jours """
    if today:
        jour = datetime.now()
        num_jour = jour.isoweekday()
    else:
        num_jour = 7
    semaine = ['lundi', 'mardi', 'mercredi', 'jeudi', 'vendredi', 'samedi', 'dimanche']
    return semaine[:num_jour]

class NavigationVisit(Action):
    """ interface de gestion des visites de sites web"""
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name', 'role'])
    name = 'navigation_visit'
    libelle = 'Visites des sites'
    description = 'Observer les sites visités'
    category = "Outils"
    request = Dict(default={}, doc="arguments de la requete en cour cote frontend", 
                    keys=['server', 'action', 'beg_filename', 'end_filename', 'beg', 'end', 'ip', 'login', 'size', 'page', 'url', 'denied'])
    form_result = Dict(default={}, doc="retour formulaire de la requete en cour coté frontend",
                        keys = ['log_request'])

    def execute(self):
        """ renvoie les données pour la mise en forme
            1 - renvoie la description du formulaire de consultation de log
            2 - valide le formulaire et renvoie les logs
            3 - repond aux requetes faites depuis un bouton (suivant, precedent, numéro de page)
        """
        params, self.server_nb = tools.get_request(self.request)
        ## 3 -
        if params.has_key('ip') or params.has_key('login') or params.has_key('page') or params.has_key('url'):
            try: 
                datas = self._get_page(params)
            except (MissingKey, MissingValue, Exception), mess:
                log.err("Erreur : navigation_visit.py, %s"%mess)
                datas = {'message':mess.message}
            return self.send_frag(datas, template='amon_navigation_visit', templates=used_templates)
        ## 2 -
        if self.form_result.has_key('log_request'):
            try:
                datas = self._valid_log_request()
            except (MissingKey, MissingValue, Exception), mess:
                log.err("Erreur : navigation_visit.py, %s"%mess)
                datas = {'message':mess.message}
            self.form_result = {}
            return self.send_frag(datas, template='amon_navigation_visit', templates=used_templates)
        ## 1 -
        result = {'titre':"Observatoire des navigations", 'sstitre1':"Choix du (des) jour(s)",
                        'sstitre2':"Critères de recherche"}
        result.update(self._get_form())
        return self.send_all(result, template='amon_navigation_visit', templates=used_templates)

    def _get_form(self):
        """ renvoie la description du formulaire de consultation de logs """
        logs = file_listing.get_logs_for_form()
        beg_day_select = {'name':"beg_day", 'libelle':"Visite du:", "options":logs}
        end_day_select = {'name':"end_day", 'libelle':"Au:", "options":logs}
        hours = [{'name':str(hour), 'libelle':"%s heure"%str(hour)}for hour in range(25)]
        hours.insert(0,{'name':'vide', 'libelle':'', 'default':'ok'})
        hour_deb = {'name':'deb_hour', 'libelle':'Entre:', 'options':[{'name':'','options':hours}]}
        hour_end = {'name':'end_hour', 'libelle':'Et:', 'options':[{'name':'','options':hours}]}
        ip = {'name':'ip', 'libelle':"Depuis l'Ip"}
        login = {'name':'login', 'libelle':'Par:'}
        nums = [{'name':str(10*num), 'libelle':str(10*num)}for num in range(6)[1:]]
        size = {'name':'size', 'libelle':'par page de', 'options':[{'name':'', 'options':nums}]}
        denied = {'name':'denied', 'libelle':'', '_libelle':'Seulement les accès refusés', 'value':'ok'}
        return dict(beg_day_select=beg_day_select, end_day_select=end_day_select, hour_deb=hour_deb, hour_end=hour_end, 
                ip=ip, login=login, size=size, denied=denied, validate = self._get_validate_btn())

### methodes qui renvoient les logs
    def _valid_log_request(self):
        """ renvoie les logs demandés depuis le retour de formulaire (page=1)"""
        result = tools.format_form_result(self.form_result['log_request'], True)
        for i in ('deb_hour', 'end_hour', 'beg_day', 'ip', 'login', 'end_day', 'size', 'denied'):
            if not result.has_key(i):
                log.err("Erreur: navigation_visit.py, il manque des données pour le parsing.")
                raise MissingValue, "Erreur : Il manque des données pour le parsing"
        ## recuperation des valeurs
        beg, end, beg_filename = result['deb_hour'].strip(), result['end_hour'].strip(), result['beg_day'].strip()
        end_filename = result['end_day'].strip()
        ip, login = result['ip'].strip(), result['login'].strip()
        size = int(result['size'].strip())
        denied = result['denied']
        if denied == False: denied = None
        else: denied = 'ok'
        ## formatage des valeurs non renseignées: vide ou '' devient None
        url = None
        result = [beg_filename, end_filename, beg, end, ip, login, url, denied, size]
        for indice, val in enumerate(result):
            if val in ['vide', '']:
                result[indice] = None
        logs, btns, num_btns = self._get_paged_logs(result)
        return dict(logs=logs, btns=btns, num_btns=num_btns)
    
    def _get_page(self, params):
        """ renvoie les logs depuis une requete classique (bouton suivant et precedent """
        if params.has_key('beg_filename') and params.has_key('end_filename'):
            beg_filename, end_filename = params['beg_filename'][0], params['end_filename'][0]
        else:
            beg_filename, end_filename = None, None
        if params.has_key('beg') and params.has_key('end'):
            beg, end =params['beg'][0], params['end'][0]
        else:
            beg, end = None, None
        if params.has_key('ip'):
            ip = params['ip'][0]
        else:
            ip = None
        if params.has_key('login'):
            login = params['login'][0]
        else:
            login = None
        if params.has_key('page'):
            page = int(params['page'][0])
        else:
            page=1
        if params.has_key('size'):
            size = int(params['size'][0])
        else:
            size = 10
        if params.has_key('url'):
            url = params['url'][0]
        else:
            url = None
        if params.has_key('denied'):
            denied = params['denied'][0]
        else:
            denied = None
        params = [beg_filename, end_filename, beg, end, ip, login, url, denied, size]
        for indice, valeur in enumerate(params):
            if valeur in ['','None', 'vide']:
                params[indice] = None
        logs, page_btns, num_btns = self._get_paged_logs(params, page)
        return dict(logs = logs, btns =page_btns, num_btns=num_btns)
#
### méthode qui renvoie les logs avec pagination
    def _get_paged_logs(self, params, page=1):
        """ renvoie les logs selon les critères demandés avec système de pagination
            params dans l'ordre: beg_filename (premier jour), end_filename (dernier jour),
                                    heure de debut, heure de fin, ip, login, url, denied, size
        """
        logs = self._get_logs(params)
        size = params[-1]
        if len(logs) > size:
            max_index = size
            if page is not None:
                max_index = size * page
            min_index = max_index - size
            length = len(logs)
            logs = logs[min_index:max_index]
            btns, num_btns = self._get_btns(params, page, length)
            return logs, btns, num_btns
        return logs, [], []

### méthode pour le parsing de log
    def _get_logs(self, params):
        """ renvoie les logs selon les critères demandés 
            params : dans l'ordre:beg_filename (premier jour), end_filename (dernier jour), heure de debut,
                                        heure de fin, ip, login, url, denied
        """
        beg_filename, end_filename, beg, end, ip, login, url, denied = params[:8]
        if beg_filename == None: 
            log.err("Erreur: navigation_visit.py, la requête doit être plus ciblée.")
            raise Exception, "Erreur : votre requête doit être plus ciblée."
        elif end_filename == None:
            days = [beg_filename]
        else:
            all_logfilenames = file_listing.get_logs()
            if all_logfilenames.index(beg_filename) < all_logfilenames.index(end_filename):
                log.err("Erreur: navigation_visit.py, le jour de début doit précéder le jour de fin.")
                raise Exception , "Erreur : Le jour de début doit précéder le jour de fin."
            days = all_logfilenames[all_logfilenames.index(end_filename):(all_logfilenames.index(beg_filename)+1)]
        logs = logparsing.get_logs(filenames=days, ip=ip,login=login, url=url, hours=(beg, end), denied=denied)
        return logs

### méthode renvoyant les boutons suivant et précédent
    def _get_btns(self, params, page=1, length=10):
        """ renvoie la description des boutons de passage de page en page.
            on passe les params de recherche 
        """
        beg_filename, end_filename, beg, end, ip, login, url, denied, size = params
        next = {'libelle': 'suivant >', 'icone':'', 'title':'',
                'href':tools.make_long_js_link(self.server_nb, self.name, balise='div_log_table_container',
                                page=page+1, beg_filename=beg_filename, end_filename=end_filename, beg=beg,
                                end=end, ip=ip, login=login, denied=denied, size=size, url=url)}
        prec = {'libelle': '< précédent', 'icone':'', 'title':'',
                'href':tools.make_long_js_link(self.server_nb, self.name, balise='div_log_table_container',
                                page=page-1, beg_filename=beg_filename, end_filename=end_filename, beg=beg,
                                end=end, ip=ip, login=login, denied=denied, size=size, url=url)}
        # calcul du nombre de page à afficher on affiche que 8 nombres donc on calcule un indice max et min 
        # ex on affiche la page 8, on aura les liens 4 5 6 7 8 9 10 11 12
        num_page = int(length/float(size)-0.1)+1
        maxindex = min(((page-1)+5), num_page)
        minindex = max(((page-1)-4), 0)
        indices = range(num_page)
        num_btns = [{'libelle':str(num+1), 'icone':'', '_class':'amon_simple_link', 'title':'page %s'%(num+1),
                    'href':tools.make_long_js_link(self.server_nb, self.name, balise='div_log_table_container',
                            page=num+1, beg_filename=beg_filename, end_filename=end_filename, beg=beg,
                            end=end, ip=ip, login=login, denied=denied, size=size, url=url)}for num in indices]
        num_btns[page-1]['href'] = ''
        num_btns[page-1]['_class'] = 'amon_simple_active_link'
        result = num_btns[minindex:maxindex]
        if minindex != 0:
            a = num_btns[0]
            a['libelle'] += "  ..."
            result.insert(0,a)
        if maxindex<num_page:
            a = num_btns[num_page-1]
            a['libelle'] = "...   %s"%a['libelle']
            result.append(a)
        if page == 1 :
            return [next], result
        elif ((page) * size)>=length:
            return [prec], result
        else:
            return [prec, next], result
    
    def _get_validate_btn(self):
       """ renvoie la description du bouton valider """
       href= tools.make_long_form_link(self.server_nb, self.name, True, ['log_request'], 'div_log_table_container')
       title='Consulter'
       libelle='Valider'
       icone='/image/ok.gif'
       return dict(href=href,title=title,libelle=libelle,icone=icone)
