# -*- coding: utf-8 -*-
###########################################################################
# Eole NG - 2007  
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr 
#  
# Action navigation_whitelist
#  
# Gère les listes blanches utilisateur (url et utilisateur)
#       
###########################################################################
""" Action de gestion des listes blanches sur amon """

from twisted.python import log
from ead2.backend.lib.action import * # import des actions et des types
from ead2.backend.actions.tools import get_request, make_form_link, make_js_link, make_key_form_link
from ead2.backend.actions.tools import format_form_result
from ead2.backend.actions.amon import amon_tools as t #fonctions de gestion de la liste blanche
from ead2.backend.actions.amon import menus
from ead2.backend.config.config import SCRIPT_DIR
from ead2.backend.config.filenames import navigation_access, dg_reload
from ead2.lib.libead import uni
import commands
from os.path import isfile, join


#nom des formulaires
form_name = {'white_user':['list_add', 'current_page'],
            'grey_list': ['list_add', 'current_page']}
used_templates = ['main', 'form', 'formselect']


class NavigationWhiteList(Action):
    """ interface de gestion des acces interneten mode liste blanche"""
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name', 'role'])
    name = 'navigation_whitelist'
    libelle = 'Mode liste blanche'
    description = 'Mode liste blanche utilisateur'
    category = None
    request = Dict(default={}, doc="arguments de la requete en cour cote frontend", 
                    keys=['server', 'action', 'current_page'])
    form_result = Dict(default={}, doc="retour formulaire de la requete en cour coté frontend",
                        keys = ['list_suppr', 'list_add', 'current_page'])
    def execute(self):
        """ renvoit les données pour la mise en forme des actions """
        params, self.server_nb = get_request(self.request)
        self.page = 'white_user'
        result = {'titre': self.description}
        if self.form_result != {}:
            if self.form_result.has_key('current_page'):
                self.page = self.form_result['current_page'][0]['value']
            try:
                self._valid_form()
            except Exception, mess:
                log.err("Erreur: navigation_visit.py, %s"%mess)
            self.form_result = {}
        if params.has_key('current_page'):
            self.page = params['current_page'][0]
        result.update(self._get_form())
        result['sstitre'] =  ''
        result['menus'] = menus._get_menu(self.server_nb, self.page)
        result['page'] = {'name':'current_page', 'default_value':self.page}
        if form_name.has_key(self.page):
            result['validate'] = {'href':make_form_link(self.server_nb, self.name, 
                                                    True, form_name[self.page]),
                                  'icone':'/image/amon/ok.gif', 
                                  'libelle':'Valider', 
                                  'title':'Valider les changements'}
            result['validation'] = make_key_form_link(self.server_nb, self.name, 
                                                    False, form_name[self.page])
        else:
            result['validate'] = ''
        return self.send_all(result, template='amon_navigation_whitelist', templates=used_templates)

    def _get_form(self):
        if self.page == 'grey_list':
            return self._get_grey_list_form()
        else:
            return self._get_white_user_form()

    def _get_grey_list_form(self):
        whitelist_site = {'name':'name','onblur':"testVoid('name', 'error_name')",
                'libelle':"Site à ajouter à la liste blanche"}
        grey_sites = [{'name':site, 'libelle':site}for site in t.get_grey_list()]
        grey_site_form = {'entete':"Retirer des sites de la liste blanche"}
        if grey_sites != []:
            grey_site_form.update({
                'validate':{'href':make_form_link(self.server_nb, self.name, 
                                            True, ['current_page','list_suppr']),
                            'icone':'/image/amon/supprimer_big.gif',
                            'libelle':'Retirer de la liste blanche',
                            'title':'Retirer les sites selectionnés de la liste blanche'},
                'select':{'name':'to_del', 
                          'options':[{'name':'site en liste blanche', 
                                                        'options':grey_sites}],
                          'libelle':"Veuillez sélectionner le ou les sites à retirer de la liste blanche",
                          'multi':'ok'}
                        })
        else:
            grey_site_form.update({"msg": "Aucun site n'est placé sur la liste blanche"})
        return dict(whitelist_site=whitelist_site, listing=grey_site_form, focus='name')

    def _get_white_user_form(self):
        whitelist_user = {'name':'name', 'onblur':"testVoid('name', 'error_name')",
                'libelle':"Login de l'utilisateur à ajouter à la liste blanche"}
        whitelist_users = [{'name':user, 'libelle':user}for user in t.get_users()['white_users']]
        whitelist_users_form = {'entete':"Utilisateurs de la liste blanche"}
        if whitelist_users != []:
            whitelist_users_form.update({
                    'validate':{'href':make_form_link(self.server_nb, self.name,
                                                    True, ['current_page','list_suppr']),
                                'icone':'/image/amon/supprimer_big.gif',
                                'libelle':'Retirer de la liste blanche',
                                'title':'Retirer les utilisateurs selectionnés de la liste blanche'},
                    'select':{'name':'to_del', 
                              'options':[{'name':'utilisateurs de la liste', 
                                            'options':whitelist_users}],
                              'libelle':"Veuillez sélectionner le ou les utilisateurs à retirer de la liste blanche",
                              'multi':'ok'}
                            })
        else:
            whitelist_users_form.update({"msg": "Aucun utilisateur n'a été placé sur liste blanche"})
        return dict(whitelist_user=whitelist_user, listing=whitelist_users_form, focus='name')

    def _valid_form(self):
        """ valide les retours de formulaire"""
        if self.page == 'white_user':
            return self._valid_white_user()
        else:
            return self._valid_grey_list()

    def _valid_white_user(self):
        """ valide l'ajout d'utilisateur à la liste blanche """
        if not self.form_result.has_key('list_add') and not self.form_result.has_key('list_suppr'):
            log.err("Erreur: il manque des données pour l'ajout/suppression d'utilisateur de la liste blanche.")
            return ''
        # on ajoute un utilisateur à la liste blanche
        if self.form_result.has_key('list_add'):
            result = self.form_result['list_add']
            user = format_form_result(result)
            username = user['name']
            t.add_user(username, 'white_user')
       # suppression d'utilisateur de la liste blanche
        elif self.form_result.has_key('list_suppr'):
            result = self.form_result['list_suppr']
            users = format_form_result(result)
            if not users.has_key('to_del'):
                return ''
            for user in users['to_del']:
                t.autorise_user(user)
        t.dansguardian_reload()

    def _valid_grey_list(self):
        """ valide l'ajout d'un site accessible pour les utilisateurs sur liste blanche"""
        if not self.form_result.has_key('list_add') and not self.form_result.has_key('list_suppr'):
            log.err("Erreur: il manque des données pour l'ajout/suppression de site accessible pour les utilisateurs whitelister.")
            return ''
        # on ajoute un site à la liste blanche
        if self.form_result.has_key('list_add'):
            result = self.form_result['list_add']
            site = format_form_result(result)
            sitename = site['name']
            t.add_site(sitename)
        # suppression d'un site de la liste blanche
        elif self.form_result.has_key('list_suppr'):
            result = self.form_result['list_suppr']
            sites = format_form_result(result)
            if not sites.has_key('to_del'):
                return ''
            t.del_site(sites['to_del'])
        t.dansguardian_reload()
