# -*- coding: utf-8 -*-
###########################################################################
# Eole NG - 2007  
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr 
#  
# Gere les groupes de machine
#  
###########################################################################
""" gere des groupes de machine definit par plage d'ips 
    propose create_group, del_group, get_all_groups, is_group, get_group, test_ip_range
"""

from os.path import isfile

from ead2.backend.actions.amon.amon_tools import add_lines, del_lines
from ead2.backend.actions.lib.ipset import time_manager
from ead2.backend.config.filenames import ipset

group_file = ipset['group']
# format du template de gestion des groupes
# un groupe c'est un nom, deux ips, un statut 0:inactif, 1:actif, un statut horaire 0:"pas d'horaire", 1:"avec horaire"
SYNTAX = "%(name)s#%(ip_from)s#%(ip_to)s#%(active)s#%(time)s"

def create_group(name, ip_from, ip_to):
    """ cree un groupe de machine nivo fichier de configuration
        @name: nom du groupe
        @ip_from: ip de début plage
        @ip_to: ip de fin de plage
    """
    if not is_group(name) and test_ip_range(ip_from, ip_to) and test_group_name(name):
        line = SYNTAX%{'name':name, 'ip_from':ip_from, 'ip_to':ip_to, 'active':'0', 'time':'0'}
        add_lines(group_file, line)
        return True
    raise Exception, "Erreur: le groupe %s existe déjà."%name

def is_group(name):
    """ vérifie l'existence d'un groupe
        @name: nom du groupe
    """
    return name in get_all_groups('name')

def get_all_groups(attr=None):
    """ renvoie la liste des groupes, attr, renvoie la liste des valeur de l'attribut attr des groupes
        @attr: nom de l'attribut
    """
    if isfile(group_file):
        if attr == 'name':
            return [group.split('#')[0] for group in file(group_file).read().splitlines()]
        elif attr == 'ip_from':
            return [group.split('#')[1] for group in file(group_file).read().splitlines()]
        elif attr == 'ip_to':
            return [group.split('#')[2] for group in file(group_file).read().splitlines()]
        elif attr == None:
            return [{'name':group.split('#')[0], 'ip_from':group.split('#')[1], 'ip_to':group.split('#')[2], 'active':group.split('#')[3], 'time':group.split('#')[4]}for group in file(group_file).read().splitlines()]
    return []

def get_group(name):
    """ renvoie les données concernant un groupe 
        @name: nom du groupe
    """
    if is_group(name):
        for group in get_all_groups():
            if group['name'] == name:
                return group
    return None

def del_group(name):
    """ supprime un groupe et ses références dans la gestion des horaires
        @name: nom du groupe
    """
    if is_group(name):
        # suprpession des horaires associés au groupe
        time_manager.del_schedule(name)
        del_only_group(name)
        return True
    raise Exception, "Erreur: le groupe %s n'existe pas."

def del_only_group(name):
    """ supprime un groupe sans toucher à sa référence horaire
    """
    if is_group(name):
        line = SYNTAX%get_group(name)
        del_lines(group_file, line)
        return True
    raise Exception, "Erreur: le groupe %s n'existe pas."

def test_ip_range(start, end):
    """ teste si le couple start, end représente bien une plage d'ips 
        @start: ip de début de plage
        @end: ip de fin de plage
    """
    start = start.split('.')
    end = end.split('.')
    if start[:-1] != end[:-1] or int(start[3]) > int(end[3]):
        raise Exception, "Erreur: les ips ne définissent pas une plage valide."
    return True

def mod_group(name, **kw):
    """ modifie un groupe 
        @name: nom du groupe
        @kw: attributs à modifier nomdel'attribut=valeur
    """
    group = get_group(name)
    for key, value in kw.items():
        if value is not None:
            group[key] = value
    del_only_group(name)
    add_lines(group_file, SYNTAX%group)
    return True

def active(name):
    """ active un groupe
        @name: nom du groupe
    """
    mod_group(name, active='1')
    return True

def unactive(name):
    """ désactive un groupe
        @name: nom du groupe
    """
    mod_group(name, active='0')
    return True

def active_time(name):
    """ active la gestion horaire pour un groupe
        @name: nom du groupe
    """
    mod_group(name, time='1')
    return True

def unactive_time(name):
    """ désactive la gestion horaire pour un groupe
        @name: nom du groupe
    """
    mod_group(name, time='0')
    return True

def test_group_name(name):
    """ teste la validité du nom d'un groupe de machine
    """
    if ' ' in name: 
        raise Exception, "Erreur : erreur le nom de groupe de machine ne doit pas comprendre d'espace"
    return True
