# -*- coding: utf-8 -*-
###########################################################################
# Eole NG - 2007  
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr 
# 
# Génère les règles iptables pour les groupes de machine
#
###########################################################################
from twisted.python import log

from ead2.backend.actions.lib.ipset import group_manager
from ead2.backend.actions.lib.ipset import time_manager
from ead2.backend.actions import tools

#### template pour les règles iptables
# partie ipset de la règle
GROUP_RULE = """-m set --set %(name)s src,dst"""

# règles avec horaires
TIME_INPUT_RULE = """/sbin/iptables -I INPUT %%s -m time --timestart %(hdeb)s --timestop %(hfin)s --days %(jour)s -j DROP"""
TIME_FORWARD_RULE = """/sbin/iptables -I FORWARD %%s -m time --timestart %(hdeb)s --timestop %(hfin)s --days %(jour)s -j DROP"""

# règle sans horaires
GROUP_INPUT_RULE = """/sbin/iptables -I INPUT %s -j DROP"""
GROUP_PREROUTING_RULE = """/sbin/iptables -t nat -I PREROUTING %s -j DROP"""

# Traduction des jours
JOUR = dict(lundi='Mon', mardi='Tue', mercredi='Wed', jeudi='Thu',
            vendredi='Fri', samedi='Sat', dimanche='Sun')

# Liste des jours
semaine = ['Mon','Tue','Wed','Thu','Fri','Sat','Sun']


class IptablesGenerator:
    def create_rules(self):
        for group in get_active_groups():
            group_datas = get_group(group)
            group_rule = GROUP_RULE%group_datas

            if group_datas.has_key('schedules'):
                for schedule in group_datas['schedules']:
                    input_rule = TIME_INPUT_RULE%schedule
                    log.msg("Execution de %s depuis iptables_generator.py"%(input_rule%group_rule))
                    tools.command_statusoutput(input_rule%group_rule)

                    forward_rule = TIME_FORWARD_RULE%schedule
                    log.msg("Execution de %s depuis iptables_generator.py"%(forward_rule%group_rule))
                    tools.command_statusoutput(forward_rule%group_rule)

            else:
                input_rule = GROUP_INPUT_RULE%group_rule
                log.msg("Execution de %s depuis iptables_generator.py"%(input_rule))
                tools.command_statusoutput(input_rule)

                prerouting_rule = GROUP_PREROUTING_RULE%group_rule
                log.msg("Execution de %s depuis iptables_generator.py"%(prerouting_rule))
                tools.command_statusoutput(prerouting_rule)

# fonctions utiles
def get_active_groups():
    """ charger la liste des groupes actifs """
    return [group for group in group_manager.get_all_groups() if group['active'] == '1']


def get_group(group):
    """ renvoie le groupe group formaté pour la mise en place des règles iptables
        @group: dico de description du groupe
    """
    group_schedules = time_manager.get_schedules(group['name'])
    if group_schedules != {}:
        if group['time'] == '1':
            group['schedules'] = format_schedules(group_schedules)
    return group

def format_schedules(schedules):
    """ formatte les plages horaires afin de minimiser le nombre de règle iptables à implémenter
    """
    formatted_schedules = []
    for jour, horaires in schedules.items():
        for horaire in horaires:
            formatted_schedules.append({'jour':JOUR[jour], 'hdeb':horaire['hdeb'], 'hfin':horaire['hfin']})
    return formatted_schedules


def get_jliste(jdeb, jfin):
    istart, iend = semaine.index(jour[jdeb]), semaine.index(jour[jfin])
    if istart < iend:
        jliste = ','.join(semaine[istart: iend+1])
    elif istart == iend:
        jliste = semaine[istart]
    else:
        jliste = ','.join(semaine[istart:])
        jliste += ','
        jliste += ','.join(semaine[:iend+1])
    return jliste

