# -*- coding: utf-8 -*-
###########################################################################
# Eole NG - 2007  
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr 
#  
# Gere les horaires pour les groupes de machine
#  
###########################################################################
""" 
Gère les horaires pour les groupes de machine
"""
from os.path import isfile
import pickle
from sets import Set

from ead2.backend.actions.amon import amon_tools 
from ead2.backend.config.filenames import ipset

# dico avec pour clé nom du groupe et pour valeur une liste de plage horaire 
# {'groupe':{'lundi':[{hdeb:'10:00', hfin:'12:00'}...], 'mardi':...}
time_file = ipset['schedule']

def get_schedules(group=None, jour=None):
    """ renvoie les horaires 
        @group: groupe dont on veut les horaires
        @jour: jour demandé (nécessite un nom de groupe)
    """
    schedules = load_schedules()
    if group:
        if schedules.has_key(group):
            if jour is not None:
                if  schedules[group].has_key(jour):
                    return schedules[group][jour]
                else:
                    return []
            else:
                return schedules[group]
        else:
            return {}
    return schedules

def add_schedule(group, jour, hdeb, hfin):
    """ crée une plage horaire 
        @group: nom du groupe
        @jour: jour de la plage
        @hdeb: heure de début de plage
        @hfin: heure de fin de plage
    """
    if test_group_schedule(group, jour, hdeb, hfin):
        schedules = get_schedules()
        if schedules.has_key(group):
            if schedules[group].has_key(jour):
                schedules[group][jour].append({'hdeb':hdeb, 'hfin':hfin})
            else:
                schedules[group][jour] = [{'hdeb':hdeb, 'hfin':hfin}]
        else:
            schedules[group] = {jour:[{'hdeb':hdeb, 'hfin':hfin}]}
        save_schedules(schedules)
        return True
    return False

def del_schedule(group, jour=None, hdeb=None, hfin=None):
    """ supprime une plage horaire ou l'ensemble des plages horaires du groupe si jour, hdeb et hfin = None
        @group: nom du groupe
        @jour: jour de la plage à supprimer
        @hdeb: heure de début de la plage
        @hfin: heure de fin de la plage
    """
    schedules = get_schedules()
    if None in [jour, hdeb, hfin]:
        if schedules.has_key(group):
            schedules.pop(group)
    else:
        schedule = {'hdeb':hdeb, 'hfin':hfin}
        if schedules.has_key(group):
            if schedules[group].has_key(jour):
                if schedule in schedules[group][jour]:
                    schedules[group][jour].remove(schedule)
    # enregistrement des modifications
    save_schedules(schedules)
    return True

def copy_schedules(group_from, group_to):
    """ copie les horaires d'un groupe à un autre
        @group_from : nom du groupe dont on copie les horaires
        @group_to : nom du groupe auquel on associe les horaires
    """
    schedules = get_schedules()
    if schedules.has_key(group_to):
        schedules.pop(group_to)
    if schedules.has_key(group_from):
        schedules[group_to] = schedules[group_from]
        save_schedules(schedules)
        return True
    raise Exception, "Erreur : le groupe %s n'a pas d'horaire configuré."

def load_schedules():
    """ charge les horaires 
    """
    if isfile(time_file):
        f = file(time_file, 'r')
        schedules = pickle.load(f)
        f.close()
        return schedules
    return {}

def save_schedules(schedules):
    """ sauvegarde les horaires """
    f = file(time_file, 'w')
    pickle.dump(schedules, f)
    f.close()
    return True

def get_time_list():
    """ récupère la liste des horaires """
    return [hour['libelle'] for hour in amon_tools.get_hours()]

def get_days_list():
    """ récupère la liste des jours """
    return [day['libelle'] for day in amon_tools.get_days()]

def test_time_range(hdeb, hfin):
    """ compare heure de début et heure de fin"""
    if int(hdeb.split(':')[0]) > int(hfin.split(':')[0]):
        raise Exception, "Erreur:L'heure de début doit précéder l'heure de fin de plage."
    elif int(hdeb.split(':')[0]) == int(hfin.split(':')[0]):
        if int(hdeb.split(':')[1]) >= int(hfin.split(':')[1]):
            raise Exception, "Erreur:L'heure de début doit précéder l'heure de fin de plage."
    return True

def test_group_schedule(group, jour, hdeb, hfin):
    """ teste la validité de la plage horaire pour le groupe groupe """
    if test_time_range(hdeb, hfin):
##        tester les intersections de plage avec des sets
        # on teste les intersections pour la nouvelle entrée horaire
        hour_list = get_time_list()
        schedule_as_set = Set(hour_list[hour_list.index(hdeb):hour_list.index(hfin)])
        for plage in get_schedules(group, jour):
            plage = Set(hour_list[hour_list.index(plage['hdeb']): hour_list.index(plage['hfin'])])
            if plage.intersection(schedule_as_set) != Set([]):
                # FIXME: on pourrait joindre les plages horaires lorsqu'elles s'intersectent
                raise Exception, "Erreur: les plages horaires doivent être distinctes (intersection vide)"
        return True
    return False
