# -*- coding: UTF-8 -*-
###########################################################################
# Eole NG - 2007  
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr 
#    
# Gère de groupe de services
#       
###########################################################################
""" gestionnaire de groupes de services """
from twisted.python import log
from ead2.backend.config.filenames import role_filename, perm_filename
from os.path import isfile
from sets import Set
from ead2.backend.config.config import module
from ead2.backend.lib.perm.parser import RoleParser, PermissionParser, Role

f_roles = RoleParser()
f_roles.parse_file(role_filename)
perms_p = PermissionParser()
perms_p.parse_file(perm_filename)


class RoleManager(PermissionParser):
    """ gère les rôles pour l'ead2
        ecriture de perm_local.ini
    """

    def __init__(self):
        PermissionParser.__init__(self)
        self.filename = perm_filename
        self.read(self.filename)

    def get_roles(self):
        """renvoie la liste des roles"""
        if not self.has_section('roles'):
            return []
        return [name for name, descr in self.items('roles')]

    def get_permissions(self):
        """renvoie la liste des permissions définies dans le .ini"""
        if not self.has_section('permissions'):
            return {}
        return dict([(action_name, [role.strip() for role in roles.split(',')if roles!=[]])for action_name, roles in self.items('permissions')])

    def get_role_perms(self, role):
        """ renvoie la liste des permissions pour le role role """
        return [name for name, roles in self.get_permissions().items() if role in roles]
    
    def add_role_perm(self, rolename, actionname):
        """ associe une action à un rôle """
        #FIXME : tester l'existence de l'action
        if not self.has_section('permissions'):
            self.add_section('permissions')
        if rolename not in self.get_roles():
            raise Exception, "Erreur: ce rôle n'existe pas."
        if self.has_option('permissions', actionname):
            if rolename in self.get_permissions()[actionname]:
                raise Exception, "Erreur : l'action est déjà associée au rôle"
            else:
                rolenames = self.get_permissions()[actionname]
                rolenames.append(rolename)
                self.remove_option('permissions', actionname)
                self.add_perm('permissions', actionname, rolenames)
                self.save()
                return True
        self.add_perm('permissions', actionname, rolename)
        self.save()
        return True

    def add_perm(self, section, option, value):
        """ ajoute une permission """
        if not self.has_section('permissions'):
            self.add_section('permissions')
        if type(value) == str:
            value = [value]
        if value == [] or value == ['']:
            return False
        self.set('permissions', option, ','.join(value))
        return True

    def add_role(self, rolename, rolelibelle):
        """ ajoute un role au fichier de permission """
        if not self.has_section('roles'):
            self.add_section('roles')
        if self.has_option('roles', rolename):
            raise Exception, "Erreur : le rôle %s existe déjà."%rolename
        self.set('roles', rolename, rolelibelle)
        self.save()
        return True

    def del_role(self, rolename):
        """ supprime un rôle """
        self.del_role_perms(rolename)
        if self.remove_section('roles', rolename):
            return True
        raise Exception, "Erreur: ce rôle n'existe pas."
        self.save()
        return True

    def del_role_perms(self, role):
        """ supprime les permissions du role role """
        for actionname in self.get_role_perms(role):
            self.del_role_perm(role, actionname)
        return True
    
    def del_role_perm(self, role, actionname):
        """ supprime la permission pour actionname du role role """
        rolenames = self.get_permissions()[actionname]
        rolenames.remove(role)
        self.remove_option('permissions', actionname)
        if rolenames != []:
            self.add_perm('permissions', actionname, rolenames)
        self.save()
        return True

    def save(self):
        """ écrit le fichier de permissions """
        f_perm = file(self.filename, 'w')
        self.write(f_perm)
        f_perm.close()

class UserRoleManager:
    """ gère l'attribution de rôle à un user (écriture d'un role_local.ini) """
    def __init__(self, filename=perm_filename):
        self.filename = filename

