# -*- coding: UTF-8 -*-
###########################################################################
# Eole NG - 2007  
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr 
#    
# Gère de groupe de services
#       
###########################################################################
""" gestionnaire de groupes de services """
from twisted.python import log
from ead2.backend.config.filenames import simple_services as services_list
from os.path import isfile
from sets import Set
from ConfigParser import ConfigParser
from ead2.backend.config.config import module


def get_available_services():
    """ renvoie la liste des services vues dans /etc/sysconfig/eole/services 
    """
    if isfile('/etc/sysconfig/eole/services'):
        tmp = file('/etc/sysconfig/eole/services', 'r')
        return [i.strip() for i in tmp.readlines()]
    return listdir('/etc/init.d/')

class ServicesParser(ConfigParser):
    """ parser pour le fichier de configuration des groupes de services """
    def parse_file(self, filename):
        """ parse un fichier """
        self.read(filename)

    def get_services(self):
        """ renvoie les groupes de services """
        if module in self.sections():
            for libelle, services in self.items(module):
                yield({'libelle':libelle, 'services':services.split(',')})

    def get_libelles(self):
        """ renvoie la liste des groupes de services """
        if module in self.sections():
            for libelle, toto in self.items(module):
                yield libelle

def get_group_services(group_name):
    """ renvoie la liste des services associés à un groupe """
    a = ServicesParser()
    a.parse_file(services_list)
    for i in a.get_services():
        if i['libelle'] == group_name:
            return i['services']
    return []

def get_groups():
    """ renvoie la liste des groupes du module """
    a = ServicesParser()
    a.parse_file(services_list)
    return list(a.get_libelles())

def get_group_available_services(group_name):
    """ renvoie la différence entre la liste des services
        du groupe et des services disponibles
    """ 
    return list(Set(get_available_services()).difference(Set(get_group_services(group_name))))

## action sur le fichier de config
def create_service_group(group_name, services):
    """ cree un groupe de service """
    if type(services) != list:
        services = [services]
    if group_name in get_groups():
        raise Exception, "Ce groupe de service existe déjà sur %s."%module
    a = ServicesParser()
    a.parse_file(services_list)
    if not a.has_section(module):
        a.add_section(module)
    a.set(module, group_name, ','.join(services))
    fh = file(services_list, 'w')
    a.write(fh)
    fh.close()

def mod_service_group(group_name, services):
    """ modifie un groupe de services """
    if type(services) != list:
        services = [services]
    if list(Set(services).difference(Set(get_group_services(group_name)))) == []:
        # pas de modification
        return True
    a = ServicesParser()
    a.parse_file(services_list)
    if not a.has_section(module):
        a.add_section(module)
    a.remove_option(module, group_name)
    a.set(module, group_name, ','.join(services))
    fh = file(services_list, 'w')
    a.write(fh)
    fh.close()

def del_service_group(group_name):
    """ supprime un groupe de services """
    if group_name not in get_groups():
        raise Exception, "Erreur : Ce groupe n'existe pas, impossible de le supprimer."
    a = ServicesParser()
    a.parse_file(services_list)
    a.remove_option(module, group_name)
    fh = file(services_list, 'w')
    a.write(fh)
    fh.close()
